from django import forms
from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import SimpleUploadedFile

from wagtail.admin.forms import WagtailAdminModelForm

import requests
from requests import HTTPError, RequestException

from danse import REQUESTS_USER_AGENT
from danse.core.models.common import VISUEL_ALLOWED_EXTENSIONS, VISUEL_MIN_SIZE
from danse.utils.wikipedia import (
    get_page_from_url,
    get_page_image,
    get_page_summary,
)

VILLE_DESCRIPTION_LIMIT = 1000


class VilleForm(WagtailAdminModelForm):
    visuel_url = forms.URLField(
        required=False,
        label="Visuel depuis la page Wikipédia",
        widget=forms.HiddenInput,
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.is_fetching = bool(self.data.get('action-fetch'))

    def clean(self):
        """Tente de télécharger le visuel si une URL est définie."""
        cleaned_data = super().clean()
        visuel_url = cleaned_data.get('visuel_url')

        if not self.is_fetching and visuel_url:
            if 'visuel' in self.errors:  # pragma: no cover
                del self._errors['visuel']

            try:
                response = requests.get(
                    visuel_url, headers={'user-agent': REQUESTS_USER_AGENT}
                )
                response.raise_for_status()

                cleaned_data['visuel'] = self.fields['visuel'].clean(
                    SimpleUploadedFile(
                        'visuel.{}'.format(visuel_url.rsplit('.', 1)[-1]),
                        response.content,
                    )
                )
            except HTTPError as e:
                self.add_error(
                    'visuel',
                    (
                        "Impossible de télécharger l'image depuis l'URL "
                        "{} (Erreur {}).".format(
                            visuel_url, e.response.status_code
                        )
                    ),
                )
            except RequestException as e:
                self.add_error(
                    'visuel',
                    (
                        "Impossible de télécharger l'image. Détails de "
                        "l'erreur : {}".format(e)
                    ),
                )
            except ValidationError as e:
                self.add_error(
                    'visuel',
                    (
                        "L'image téléchargée depuis la page Wikipedia est "
                        "invalide. {}".format(' '.join(list(e)))
                    ),
                )

        return cleaned_data

    def _post_clean(self):
        """Tente de récupérer les informations depuis la page Wikipedia."""
        if not self.is_fetching:
            return super()._post_clean()

        try:
            page = self.get_wikipedia_page(
                self.cleaned_data.get('url_wikipedia')
            )
        except ValidationError as e:
            self.add_error('url_wikipedia', e)
        else:
            self.data = self.data.copy()

            if 'description' in self.errors:  # pragma: no cover
                del self._errors['description']

            # récupère et définis la description
            self.data['description'] = self.get_description_from_page(page)
            self.cleaned_data['description'] = self.data['description']

            if 'visuel' in self.errors:  # pragma: no cover
                del self._errors['visuel']

            # récupère et définis l'URL du visuel
            self.data['visuel_url'] = self.get_visuel_url_from_page(page)
            self.cleaned_data['visuel_url'] = self.data['visuel_url']

            self.add_error(
                None,
                (
                    "Veuillez vérifier les informations récupérées "
                    "depuis Wikipedia avant d'enregistrer."
                ),
            )

    def get_wikipedia_page(self, url_wikipedia):
        if not url_wikipedia:
            raise ValidationError(
                (
                    "Ce champ est requis pour pouvoir récupérer les "
                    "informations depuis Wikipedia."
                ),
                code='required',
            )

        page = get_page_from_url(url_wikipedia)

        if not page.exists():
            raise ValidationError(
                "Ce lien ne pointe pas vers une page Wikipedia existante.",
                code='unknown',
            )

        return page

    def get_description_from_page(self, page):
        return self.fields['description'].widget.format_value(
            get_page_summary(page, VILLE_DESCRIPTION_LIMIT)
        )

    def get_visuel_url_from_page(self, page):
        try:
            page_image = get_page_image(page)['original']
        except (ValueError, KeyError):
            self.add_error('visuel_url', "Aucune image ne semble être définie.")
            return ''

        # vérifie l'extension de l'image
        page_image_ext = page_image['source'].rsplit('.', 1)[1].lower()
        if page_image_ext not in VISUEL_ALLOWED_EXTENSIONS:
            self.add_error(
                'visuel_url', "Le format de l'image n'est pas valide."
            )
            return ''

        # vérifie les dimensions de l'image
        if (
            page_image['width'] < VISUEL_MIN_SIZE[0]
            or page_image['height'] < VISUEL_MIN_SIZE[1]
        ):
            self.add_error(
                'visuel_url', "La taille de l'image est trop petite."
            )
            return ''

        return page_image['source']


class EvenementForm(WagtailAdminModelForm):
    def __init__(self, *args, can_choose_structure=True, **kwargs):
        if not can_choose_structure:
            self.base_fields = self.base_fields.copy()
            self.base_fields.pop('structure')
            self._meta.fields.remove('structure')

        super().__init__(*args, **kwargs)

        if can_choose_structure:
            self.fields['lieu'].required = False
            self.fields['lieu'].help_text = (
                "Laissez vide pour utiliser le même lieu que celui de la "
                "structure sélectionnée."
            )

    def clean(self):
        cleaned_data = super().clean()

        if 'structure' in self.fields:
            structure = cleaned_data.get('structure')

            if cleaned_data.get('lieu') is None:
                if structure is None or structure.lieu is None:
                    self.add_error(
                        'lieu',
                        (
                            "Veuillez sélectionner un lieu ou une structure "
                            "avec un lieu de défini."
                        ),
                    )
                else:
                    cleaned_data['lieu'] = structure.lieu

        return cleaned_data
