from django import forms
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe

from wagtail.admin.edit_handlers import (
    FieldPanel,
    FieldRowPanel,
    HelpPanel,
    MultiFieldPanel,
)

from danse.utils.widgets import DurationWidget, GeocodedPointWidget

from .widgets import (
    FestivalChooser,
    LieuChooser,
    StructureChooser,
    VilleChooser,
)


class VilleVisuelPanel(FieldPanel):
    field_template = 'admin/edit_handlers/ville_visuel_panel.html'

    def render_as_field(self):
        return mark_safe(
            render_to_string(
                self.field_template,
                {
                    'field': self.bound_field,
                    'field_type': self.field_type(),
                    'visuel_url_field': self.form['visuel_url'],
                },
            )
        )


evenement_panels = [
    FieldPanel('nom', classname='title full'),
    MultiFieldPanel(
        [
            FieldPanel('genre'),
            FieldPanel('artiste'),
            FieldPanel('description'),
            FieldPanel('visuel'),
            FieldPanel('url_externe'),
        ],
        heading="Description",
    ),
    MultiFieldPanel(
        [
            FieldRowPanel(
                [
                    FieldPanel('debut'),
                    FieldPanel('duree', widget=DurationWidget),
                ]
            )
        ],
        heading="Date et heure",
    ),
    MultiFieldPanel(
        [
            FieldPanel('structure', widget=StructureChooser),
            FieldPanel('lieu', widget=LieuChooser),
            FieldPanel('age_min'),
            FieldPanel('festival', widget=FestivalChooser),
        ],
        heading="Informations pratiques",
    ),
    MultiFieldPanel(
        [
            FieldRowPanel(
                [FieldPanel('tarif_plein'), FieldPanel('tarif_reduit')]
            ),
            FieldPanel('url_billetterie'),
        ],
        heading="Tarif",
    ),
]

festival_panels = [
    FieldPanel('nom', classname='title full'),
    MultiFieldPanel(
        [
            FieldPanel('description'),
            FieldPanel('visuel'),
            FieldPanel('url_externe'),
        ],
        heading="Description",
    ),
    MultiFieldPanel(
        [FieldRowPanel([FieldPanel('debut'), FieldPanel('fin')])],
        heading="Date",
    ),
    HelpPanel(
        template='admin/edit_handlers/festival_evenements_panel.html',
        heading="Évènements",
    ),
]

lieu_panels = [
    FieldPanel('nom', classname='title full'),
    MultiFieldPanel(
        [
            FieldPanel('rue'),
            FieldPanel('ville', widget=VilleChooser),
            FieldPanel(
                'geom',
                widget=GeocodedPointWidget(
                    options={'searchParamsMap': {'q': ['nom', 'ville']}},
                    attrs={'disabled': True},
                ),
            ),
            HelpPanel(
                content=(
                    "La géolocalisation du lieu est recherchée à partir "
                    "du nom et de la ville lorsqu'ils sont modifiés. Si "
                    "la position ne correspond pas ou n'est pas trouvée, "
                    "vous pouvez déplacer le point sur la carte. Il est "
                    "recommandé dans ce cas de mettre à jour ce lieu "
                    "dans <a href=\"https://www.openstreetmap.org/\">"
                    "OpenStreetMap</a>."
                ),
                template='admin/edit_handlers/help_block.html',
                classname='info',
            ),
        ],
        heading="Localisation",
    ),
]

structure_panels = [
    FieldPanel('nom', classname='title full'),
    MultiFieldPanel(
        [
            FieldPanel('accroche'),
            FieldPanel('description'),
            FieldPanel('visuel'),
            FieldPanel('lieu', widget=LieuChooser),
        ],
        heading="Description",
    ),
    MultiFieldPanel(
        [
            FieldPanel('email'),
            FieldPanel('telephone'),
            FieldPanel('url_site'),
            FieldPanel('url_billetterie'),
        ],
        heading="Coordonnées",
    ),
]

ville_panels = [
    MultiFieldPanel(
        [
            FieldRowPanel(
                [
                    FieldPanel('nom', classname='col8'),
                    FieldPanel('code_postal', classname='col4'),
                ]
            ),
            FieldPanel(
                'geom',
                widget=GeocodedPointWidget(
                    options={
                        'readOnly': True,
                        'searchParams': {'country': 'France'},
                        'searchParamsMap': {
                            'city': 'nom',
                            'postalcode': 'code_postal',
                        },
                    },
                ),
            ),
            HelpPanel(
                content=(
                    "La géolocalisation de la ville est automatiquement "
                    "recherchée à partir du nom et du code postal "
                    "lorsqu'ils sont modifiés. Si la position ne "
                    "correspond pas ou n'est pas trouvée, veuillez "
                    "vérifier ces champs."
                ),
                template='admin/edit_handlers/help_block.html',
                classname='info',
            ),
        ],
        heading="Localisation",
    ),
    MultiFieldPanel(
        [
            FieldPanel('description'),
            VilleVisuelPanel('visuel'),
            FieldPanel(
                'url_wikipedia',
                widget=forms.URLInput(
                    attrs={'placeholder': 'https://fr.wikipedia.org/wiki/'}
                ),
            ),
        ],
        heading="Informations",
    ),
]

consigne_panels = [
    FieldPanel(
        'consigne',
        classname='title',
        widget=forms.Textarea(attrs={'rows': '5'}),
    )
]
