from wagtail.core.permission_policies import ModelPermissionPolicy

from danse.core.models import Evenement, Festival, Lieu, Structure, Ville
from danse.jouer.models import Consigne


class EvenementPermissionPolicy(ModelPermissionPolicy):
    def user_has_permission(self, user, action):
        """
        Restreins les permissions si le compte utilisateur est associé à une
        structure qui n'existe pas encore.
        """
        return super().user_has_permission(user, action) and (
            not user.is_structure_user or user.structure_id is not None
        )

    def user_has_permission_for_instance(self, user, action, instance):
        """
        Étends les permissions si la structure de l'évènement donné est
        associée au compte utilisateur et qu'il peut créer des évènements.
        """
        if (
            action in ('change', 'delete')
            and user.is_structure_user
            and user.structure_id == instance.structure_id
        ):
            return user.has_perm(self._get_permission_name('add'))
        return super().user_has_permission_for_instance(user, action, instance)


class StructurePermissionPolicy(ModelPermissionPolicy):
    def user_has_permission(self, user, action):
        """
        Étends les permissions d'ajout si le compte utilisateur est associé
        à une structure qui n'existe pas encore.
        """
        return super().user_has_permission(user, action) or (
            action == 'add'
            and user.is_structure_user
            and user.structure_id is None
        )

    def user_has_permission_for_instance(self, user, action, instance):
        """
        Étends les permissions de modification si le compte utilisateur est
        associé à la structure donnée.
        """
        if (
            action == 'change'
            and user.is_structure_user
            and user.structure_id == instance.pk
        ):
            return True
        return super().user_has_permission_for_instance(user, action, instance)


evenement_permission_policy = EvenementPermissionPolicy(Evenement)
festival_permission_policy = ModelPermissionPolicy(Festival)
lieu_permission_policy = ModelPermissionPolicy(Lieu)
structure_permission_policy = StructurePermissionPolicy(Structure)
ville_permission_policy = ModelPermissionPolicy(Ville)

consigne_permission_policy = ModelPermissionPolicy(Consigne)
