import datetime
import json
import re
from unittest.mock import patch

from django.conf import settings
from django.contrib.messages import ERROR as ERROR_LEVEL
from django.contrib.messages import SUCCESS as SUCCESS_LEVEL
from django.core.exceptions import NON_FIELD_ERRORS
from django.urls import reverse
from django.utils.html import escape

from wagtail.core.models import ModelLogEntry

import pytest
import requests
from bs4 import BeautifulSoup
from factory.django import ImageField
from freezegun import freeze_time
from pytest_django.asserts import assertRedirects
from webtest import Form, Upload

from danse.core import models
from danse.core.tests.factories import (
    EvenementFactory,
    FestivalFactory,
    LieuFactory,
    StructureFactory,
    VilleFactory,
)
from danse.jouer.models import Consigne, JeuSettings
from danse.jouer.tests.factories import ConsigneFactory
from danse.users.tests.factories import GroupFactory
from danse.utils.tests import (
    WAGTAILHOME_URL,
    AdminViewMixin,
    ViewMixin,
    ViewSetMixin,
    assertRedirectsToHome,
)


def create_image(**kwargs):
    """Crée une image avec Factory Boy."""
    kwargs.setdefault('width', 1350)
    return ImageField()._make_data(kwargs)


class TestHomepageSummary(ViewMixin):
    url = WAGTAILHOME_URL
    evenements_index_url = reverse('admin_evenement:index')
    structures_index_url = reverse('admin_structure:index')

    def test_summary_admin(self, admin_user):
        StructureFactory()
        EvenementFactory.create_batch(3)

        stats = self.get(user=admin_user).html.select_one('.stats')
        # il y a 3 évènements au total
        assert stats.find('a', href=self.evenements_index_url).find(
            'span', text="3"
        )
        # il y a 4 structures au total
        assert stats.find('a', href=self.structures_index_url).find(
            'span', text="4"
        )

    def test_summary_structure_user(self, structure_user):
        structure = StructureFactory(nom="Déstructure")
        EvenementFactory.create_batch(2, structure=structure)
        EvenementFactory.create_batch(3)

        stats = self.get(user=structure_user).html.select_one('.stats')
        # il n'y a pas d'évènements visible
        assert stats.find('a', href=self.evenements_index_url).find(
            'span', text="0"
        )
        # la structure n'est pas encore créée
        assert "Structure" not in stats.prettify()

        structure_user.structure = structure
        structure_user.save()

        structure_edit_url = reverse(
            'admin_structure:edit', args=(structure.pk,)
        )

        stats = self.get(user=structure_user).html.select_one('.stats')
        # il y a 2 évènements de la structure
        assert stats.find('a', href=self.evenements_index_url).find(
            'span', text="2"
        )
        # la structure associée est visible et modifiable
        assert "Structure" in stats.prettify()
        assert stats.find('a', href=structure_edit_url).find(
            'span', text="Déstructure"
        )


class TestVilleAdminView(ViewSetMixin):
    factory = VilleFactory
    model = models.Ville
    namespace = 'admin_ville'

    def test_create_edit(self, faker):
        response = self.get(self.create_url)
        assert "Ajouter une ville" in response
        form = response.forms[1]
        assert form.action == self.create_url

        # on crée une instance
        form['nom'] = "Libreville"
        form['code_postal'] = "01234"
        form['geom'] = faker.geo_point().geojson
        form['description'] = (
            response.context['form']
            .fields['description']
            .widget.format_value("<p>Une description de la ville.</p>")
        )
        form['visuel'] = Upload('visuel.png', create_image())
        form['url_wikipedia'] = "https://fr.wikipedia.org/wiki/Libreville"
        assert "Libreville" in form.submit(status=302).follow()

        instance = self.model.objects.get(nom="Libreville")

        # on peut modifier l'instance
        self.get(self.get_edit_url(instance.pk), status=200)

    def test_not_allowed_visuel_extension(self, faker):
        form = self.get(self.create_url).forms[1]

        # on définis une ville avec une image au format TIFF
        form['nom'] = "Libreville"
        form['code_postal'] = "01234"
        form['visuel'] = Upload('visuel.tiff', create_image(format='TIFF'))

        errors = form.submit(status=200).context['form'].errors
        assert "jpg, jpeg, png" in errors['visuel'][0]

    def test_fetch_and_create(self, faker, requests_mock):
        form = self.get(self.create_url).forms[1]
        assert form['url_wikipedia'].attrs['placeholder'] == (
            'https://fr.wikipedia.org/wiki/'
        )

        # on définis une ville avec une URL
        form['nom'] = "Libreville"
        form['code_postal'] = "01234"
        form['geom'] = faker.geo_point().geojson
        form['description'] = "null"
        form['url_wikipedia'] = "https://fr.wikipedia.org/wiki/Ville"

        with patch('danse.admin.forms.get_page_from_url') as mock_page:
            mock_page.return_value.exists.return_value = True

            with patch('danse.admin.forms.get_page_summary') as mock_summary:
                mock_summary.return_value = (
                    "<h1>Ville</h1><p>Une description.</p>"
                )

                with patch('danse.admin.forms.get_page_image') as mock_image:
                    mock_image.return_value = {
                        'name': 'Libreville.jpg',
                        'original': {
                            'source': 'http://localhost/media/Libreville.jpg',
                            'width': 1400,
                            'height': 700,
                        },
                    }

                    # ... et on récupère ses informations depuis Wikipedia
                    response = form.submit('action-fetch', status=200)
                    assert "vérifier les informations récupérées" in response
                    assert escape("du fait d'erreurs") not in response
                    assert set(response.context['form'].errors.keys()) == {
                        NON_FIELD_ERRORS
                    }

        assert not self.model.objects.filter(nom="Libreville").exists()

        # ... qu'on enregistre désormais
        form = response.forms[1]
        requests_mock.get(
            form['visuel_url'].value,
            content=create_image(width=1400, height=700),
        )
        form.submit(status=302).follow()

        ville = self.model.objects.get(nom="Libreville")
        # FIXME https://github.com/wagtail/wagtail/issues/7341
        assert (
            re.fullmatch(
                (
                    r'<p data-block-key=".+">Ville</p>'
                    r'<p data-block-key=".+>Une description.</p>'
                ),
                ville.description,
            )
            is not None
        )
        assert ville.visuel.width == 1400 and ville.visuel.height == 700
        assert ville.url_wikipedia == "https://fr.wikipedia.org/wiki/Ville"

    def test_edit_and_fetch_url_required(self):
        ville = VilleFactory()

        form = self.get(self.get_edit_url(ville.pk)).forms[1]
        form['url_wikipedia'] = ''

        response = form.submit('action-fetch', status=200)
        errors = response.context['form'].errors
        assert 'url_wikipedia' in errors
        assert "champ est requis" in str(errors['url_wikipedia'])

    def test_edit_and_fetch_page_not_exists(self):
        ville = VilleFactory()

        form = self.get(self.get_edit_url(ville.pk)).forms[1]
        form['url_wikipedia'] = "https://fr.wikipedia.org/wiki/Ville"

        with patch('danse.admin.forms.get_page_from_url') as mock_page:
            mock_page.return_value.exists.return_value = False

            response = form.submit('action-fetch', status=200)

        assert escape("du fait d'erreurs") in response
        errors = response.context['form'].errors
        assert 'url_wikipedia' in errors
        assert "page Wikipedia existante" in str(errors['url_wikipedia'])

    def test_edit_and_fetch_page_image_error(self):
        ville = VilleFactory(visuel='/tmp/does-not-exist.jpg')

        form = self.get(self.get_edit_url(ville.pk)).forms[1]
        form['url_wikipedia'] = "https://fr.wikipedia.org/wiki/Ville"

        with patch('danse.admin.forms.get_page_from_url') as mock_page:
            mock_page.return_value.exists.return_value = True

            with patch('danse.admin.forms.get_page_summary') as mock_summary:
                mock_summary.return_value = "<p>Une description.</p>"

                with patch('danse.admin.forms.get_page_image') as mock_image:
                    mock_image.return_value = {}

                    # aucune image n'est disponible pour la page Wikipedia
                    response = form.submit('action-fetch', status=200)
                    errors = response.context['form'].errors
                    assert "Aucune image" in str(errors['visuel_url'])

                    mock_image.return_value = {
                        'name': 'Libreville.jpg',
                        'original': {
                            'source': 'http://localhost/media/Libreville.tiff',
                            'width': 1400,
                            'height': 700,
                        },
                    }

                    # le format de l'image de la page n'est pas supporté
                    response = form.submit('action-fetch', status=200)
                    errors = response.context['form'].errors
                    assert "Le format" in str(errors['visuel_url'])

                    mock_image.return_value = {
                        'name': 'Libreville.jpg',
                        'original': {
                            'source': 'http://localhost/media/Libreville.jpg',
                            'width': 200,
                            'height': 100,
                        },
                    }

                    # la taille de l'image de la page est trop petite
                    response = form.submit('action-fetch', status=200)
                    errors = response.context['form'].errors
                    assert "La taille" in str(errors['visuel_url'])

    def test_edit_and_download_visuel_error(self, requests_mock):
        ville = VilleFactory()
        visuel_url = 'http://localhost/media/Libreville.jpg'

        form = self.get(self.get_edit_url(ville.pk)).forms[1]
        form['visuel_url'] = visuel_url

        requests_mock.get(visuel_url, exc=requests.ConnectTimeout)
        errors = form.submit(status=200).context['form'].errors
        assert "Impossible de télécharger" in str(errors['visuel'])

        requests_mock.get(visuel_url, status_code=404)
        errors = form.submit(status=200).context['form'].errors
        assert "Erreur 404" in str(errors['visuel'])

        requests_mock.get(visuel_url, content=b'')
        errors = form.submit(status=200).context['form'].errors
        assert "invalide" in str(errors['visuel'])

    def test_delete_protected(self):
        ville = VilleFactory()
        LieuFactory(nom="Un lieu", ville=ville)

        response = (
            self.get(self.get_delete_url(ville.pk), status=200)
            .forms[1]
            .submit(status=200)
        )
        assert "Un lieu" in response


class TestVilleChooserView(AdminViewMixin):
    url = reverse('ville_chooser:choose')

    def test_search(self):
        VilleFactory(nom="Foo")
        VilleFactory(nom="Bar")
        VilleFactory(nom="Foobar")

        response = self.get(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 3

        form = Form(response, str(html.select_one('form')), 'html.parser')
        form['q'] = 'bar'
        response = form.submit(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 2


class TestLieuAdminView(ViewSetMixin):
    factory = LieuFactory
    model = models.Lieu
    namespace = 'admin_lieu'

    def test_create_edit(self, faker):
        ville = VilleFactory()

        response = self.get(self.create_url)
        assert "Ajouter un lieu" in response
        form = response.forms[1]
        assert form.action == self.create_url

        # on crée une instance
        form['nom'] = "Olieux"
        form['rue'] = "13 rue des marguerites"
        form['ville'] = ville.id
        form['geom'] = faker.geo_point().geojson
        assert "Olieux" in form.submit(status=302).follow()

        instance = self.model.objects.get(nom="Olieux", ville=ville)

        assert ModelLogEntry.objects.for_instance(instance).get().action == (
            'wagtail.create'
        )

        # on peut modifier l'instance
        self.get(self.get_edit_url(instance.pk), status=200)


class TestLieuChooserView(AdminViewMixin):
    url = reverse('lieu_chooser:choose')

    def test_search(self):
        LieuFactory(nom="Foo")
        LieuFactory(nom="Bar")
        LieuFactory(nom="Foobar")

        response = self.get(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 3

        form = Form(response, str(html.select_one('form')), 'html.parser')
        form['q'] = 'bar'
        response = form.submit(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 2


class TestStructureAdminView(ViewSetMixin):
    factory = StructureFactory
    model = models.Structure
    namespace = 'admin_structure'

    def test_create_edit(self):
        lieu = LieuFactory()

        response = self.get(self.create_url)
        assert "Ajouter une structure" in response
        form = response.forms[1]
        assert form.action == self.create_url

        # on crée une instance
        form['nom'] = "Théâtre des Olieux"
        form['accroche'] = "Une scène de la région"
        form['description'] = (
            response.context['form']
            .fields['description']
            .widget.format_value("<p>Une description de la structure.</p>")
        )
        form['visuel'] = Upload('visuel.png', create_image())
        form['lieu'] = lieu.id
        form['email'] = "contact@olieux.fr"
        form['telephone'] = "0987654321"
        assert "Théâtre des Olieux" in form.submit(status=302).follow()

        instance = self.model.objects.get(nom="Théâtre des Olieux")
        assert instance.telephone.as_national == "09 87 65 43 21"

        assert ModelLogEntry.objects.for_instance(instance).get().action == (
            'wagtail.create'
        )

        # on peut modifier l'instance
        self.get(self.get_edit_url(instance.pk), status=200)

    def test_create_edit_user_unauthorized(self, user):
        assertRedirectsToHome(self.get(self.create_url, user=user))
        assertRedirectsToHome(
            self.get(self.get_edit_url(StructureFactory().pk), user=user)
        )

    def test_create_edit_structure_user(self, structure_user):
        lieu = LieuFactory()

        # on peut créer une structure si elle n'existe pas
        assert structure_user.structure_id is None
        response = self.get(self.create_url, user=structure_user, status=200)

        form = response.form
        form['nom'] = "Théâtre des Olieux"
        form['accroche'] = "Une scène de la région"
        form['description'] = (
            response.context['form']
            .fields['description']
            .widget.format_value("<p>Une description de la structure.</p>")
        )
        form['visuel'] = Upload('visuel.png', create_image())
        form['lieu'] = lieu.id
        form.submit(status=302).follow()

        # la structure est associée au compte utilisateur
        structure_user.refresh_from_db()
        assert structure_user.structure_id is not None

        # on peut modifier sa structure
        self.get(
            self.get_edit_url(structure_user.structure.pk),
            user=structure_user,
            status=200,
        )
        # … et uniquement la sienne
        assertRedirectsToHome(
            self.get(
                self.get_edit_url(StructureFactory().pk), user=structure_user
            )
        )
        # … mais on ne peut plus en créer
        assertRedirectsToHome(self.get(self.create_url, user=structure_user))


class TestStructureChooserView(AdminViewMixin):
    url = reverse('structure_chooser:choose')

    def test_search(self):
        StructureFactory(nom="Foo")
        StructureFactory(nom="Bar")
        StructureFactory(nom="Foobar")

        response = self.get(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 3

        form = Form(response, str(html.select_one('form')), 'html.parser')
        form['q'] = 'bar'
        response = form.submit(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 2


class TestFestivalAdminView(ViewSetMixin):
    factory = FestivalFactory
    model = models.Festival
    namespace = 'admin_festival'

    @freeze_time('2020-01-10')
    def test_index(self):
        FestivalFactory(debut_str='2020-01-01', fin_str='2020-01-15')
        FestivalFactory(debut_str='2020-01-17', fin_str='2020-01-20')
        FestivalFactory(debut_str='2020-01-18', fin_str='2020-01-20')

        html = self.get(self.index_url, status=200).html
        assert len(html.select('.listing tbody tr')) == 2
        assert html.select_one('input#id_debut').attrs['value'] == '2020-01-10'

    @freeze_time('2020-01-10')
    def test_index_filter(self):
        FestivalFactory(debut_str='2020-01-01', fin_str='2020-01-05')

        response = self.get(self.index_url, status=200)
        assert len(response.html.select('.listing tbody tr')) == 0

        # on filtre sur la date
        form = response.forms[2]
        form['debut'] = '2020-01-01'
        response = form.submit(status=200)
        assert len(response.html.select('.listing tbody tr')) == 1

    def test_create_edit(self):
        response = self.get(self.create_url)
        assert "Ajouter un festival" in response
        form = response.forms[1]
        assert form.action == self.create_url

        # on crée une instance
        form['nom'] = "Festival #1"
        form['description'] = (
            response.context['form']
            .fields['description']
            .widget.format_value("<p>Une description du festival.</p>")
        )
        form['visuel'] = Upload('visuel.png', create_image())
        form['debut'] = "2020-01-17"
        form['fin'] = "2020-01-19"
        assert "Festival #1" in form.submit(status=302).follow()

        instance = self.model.objects.get(nom="Festival #1")

        # on peut modifier l'instance
        response = self.get(self.get_edit_url(instance.pk), status=200)
        assert "Il n'y a aucun évènement pour le moment." in response

    def test_edit_with_evenements(self):
        festival = FestivalFactory()
        EvenementFactory.reset_sequence()
        EvenementFactory.create_batch(3, festival=festival)

        html = self.get(self.get_edit_url(festival.pk), status=200).html
        links = html.find_all('a', title="Modifier l'évènement")
        assert len(links) == 3
        assert links[0].text.strip() == "Évènement #0"


class TestFestivalChooserView(AdminViewMixin):
    url = reverse('festival_chooser:choose')

    def test_search(self):
        FestivalFactory(nom="Foo")
        FestivalFactory(nom="Bar")
        FestivalFactory(nom="Foobar")

        response = self.get(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 3

        form = Form(response, str(html.select_one('form')), 'html.parser')
        form['q'] = 'bar'
        response = form.submit(status=200)
        html = BeautifulSoup(response.json['html'], 'html.parser')
        assert len(html.select('.listing tbody tr')) == 2


class TestEvenementAdminView(ViewSetMixin):
    factory = EvenementFactory
    model = models.Evenement
    namespace = 'admin_evenement'

    @freeze_time('2020-01-10')
    def test_index(self):
        EvenementFactory(debut_str='2020-01-01 18:00')
        EvenementFactory(debut_str='2020-01-17 18:00')
        EvenementFactory(debut_str='2020-01-18 18:00')

        html = self.get(self.index_url, status=200).html
        assert len(html.select('.listing tbody tr')) == 2
        assert html.select('.listing thead th')[2].text.strip() == "Structure"
        assert html.select_one('input#id_debut').attrs['value'] == '2020-01-10'
        assert html.select_one('select#id_lieu')
        assert html.select_one('select#id_structure')

    @freeze_time('2020-01-10')
    def test_index_structure_user(self, structure_user):
        evenement = EvenementFactory()
        EvenementFactory.create_batch(2)

        # on ne peut accèder tant que la structure n'est pas définie
        assertRedirectsToHome(self.get(self.index_url, user=structure_user))

        # on associe la structure d'un évènement au compte
        structure_user.structure = evenement.structure
        structure_user.save()

        html = self.get(self.index_url, user=structure_user, status=200).html
        # on peut désormais créer des évènements et voir le sien
        assert html.find('a', href=self.create_url)
        assert len(html.select('.listing tbody tr')) == 1
        assert html.select('.listing thead th')[2].text.strip() == "Lieu"
        assert html.select_one('select#id_lieu')
        assert not html.select_one('select#id_structure')

    @freeze_time('2020-01-10')
    def test_index_filter(self):
        EvenementFactory(debut_str='2020-01-01 18:00')

        response = self.get(self.index_url, status=200)
        assert len(response.html.select('.listing tbody tr')) == 0

        # on filtre sur la date
        form = response.forms[2]
        form['debut'] = '2020-01-01'
        response = form.submit(status=200)
        assert len(response.html.select('.listing tbody tr')) == 1

    @freeze_time('2020-01-10')
    def test_index_search(self):
        EvenementFactory(nom="Foo")
        EvenementFactory(nom="Bar")
        EvenementFactory(nom="Foobar")

        response = self.get(self.index_url, status=200)
        assert len(response.html.select('.listing tbody tr')) == 3

        form = response.forms[1]
        form['q'] = 'bar'
        response = form.submit(status=200)
        assert len(response.html.select('.listing tbody tr')) == 2

    def test_create_edit(self):
        structure = StructureFactory()

        response = self.get(self.create_url)
        assert escape("Ajouter un évènement") in response
        form = response.forms[1]
        assert form.action == self.create_url

        # on crée une instance
        form['nom'] = "Évènement #1"
        form['genre'].select(text='Spectacle')
        form['artiste'] = "Une ou un artiste"
        form['description'] = (
            response.context['form']
            .fields['description']
            .widget.format_value("<p>Une description du spectacle.</p>")
        )
        form['visuel'] = Upload('visuel.png', create_image())
        form['debut'] = "2018-01-18 20:00"
        form['duree_0'] = "2"
        form['structure'] = structure.id
        assert "Évènement #1" in form.submit(status=302).follow()

        instance = self.model.objects.get(nom="Évènement #1")
        assert instance.duree == datetime.timedelta(hours=2)
        assert list(structure.lieu.evenements.all()) == [instance]

        assert ModelLogEntry.objects.for_instance(instance).get().action == (
            'wagtail.create'
        )

        festival = FestivalFactory()

        # on modifie l'instance
        form = self.get(self.get_edit_url(instance.pk), status=200).forms[1]
        form['nom'] = "Évènement modifié"
        form['festival'] = festival.id
        response = form.submit(status=200)
        # ... la date de l'évènement est en dehors du festival
        assert 'festival' in response.context['form'].errors

        form = response.forms[1]
        form['debut'] = "2020-01-18 20:00"
        assert "Évènement modifié" in form.submit(status=302).follow()

        instance.refresh_from_db()
        assert instance.nom == "Évènement modifié"
        assert list(festival.evenements.all()) == [instance]

        ModelLogEntry.objects.for_instance(instance).get(action='wagtail.edit')

    def test_create_invalid_lieu(self):
        response = self.get(self.create_url).forms[1].submit(status=200)
        # le lieu est requis si la structure est nulle
        assert 'lieu' in response.context['form'].errors
        assert 'structure' in response.context['form'].errors

        structure = StructureFactory(lieu=None)

        form = response.forms[1]
        form['structure'] = structure.id
        response = form.submit(status=200)
        # le lieu est requis si la structure n'a pas de lieu
        assert 'lieu' in response.context['form'].errors
        assert 'structure' not in response.context['form'].errors

    def test_create_edit_user_unauthorized(self, user, structure_group):
        assertRedirectsToHome(self.get(self.create_url, user=user))
        assertRedirectsToHome(
            self.get(self.get_edit_url(EvenementFactory().pk), user=user)
        )

        user.groups.add(structure_group)
        assert user.has_perm('core.add_evenement')
        assert not user.has_perm('core.change_evenement')
        self.get(self.create_url, user=user, status=200)
        assertRedirectsToHome(
            self.get(self.get_edit_url(EvenementFactory().pk), user=user)
        )

    def test_create_edit_user_with_permissions(self, user):
        user.groups.add(
            GroupFactory(
                permissions__codenames=['add_evenement', 'change_evenement']
            )
        )
        assert user.has_perm('core.add_evenement')
        assert user.has_perm('core.change_evenement')
        form = self.get(self.create_url, user=user, status=200).form
        assert 'structure' in form.fields

        evenement = EvenementFactory()
        self.get(self.get_edit_url(evenement.pk), user=user, status=200)

    def test_create_structure_user_without_structure(self, structure_user):
        assert structure_user.structure_id is None
        assertRedirectsToHome(self.get(self.create_url, user=structure_user))

    def test_create_edit_structure_user(self, structure_user, structure_group):
        structure = StructureFactory()
        structure_user.structure = structure
        structure_user.save()

        response = self.get(self.create_url, user=structure_user)
        form = response.form
        assert 'structure' not in form.fields
        assert form['lieu'].value == str(structure.lieu_id)

        # on crée un évènement pour la structure
        form['nom'] = "Évènement #1"
        form['genre'].select(text='Spectacle')
        form['artiste'] = "Une ou un artiste"
        form['description'] = (
            response.context['form']
            .fields['description']
            .widget.format_value("<p>Une description du spectacle.</p>")
        )
        form['visuel'] = Upload('visuel.png', create_image())
        form['debut'] = "2018-01-18 20:00"
        form['duree_0'] = "2"
        form.submit(status=302).follow()

        # la structure de l'évènement est celle du compte utilisateur
        evenement = self.model.objects.get(nom="Évènement #1")
        assert evenement.structure == structure_user.structure

        # on peut modifier et supprimer l'évènement pour sa structure
        edit_url = self.get_edit_url(evenement.pk)
        self.get(edit_url, user=structure_user, status=200)
        delete_url = self.get_delete_url(evenement.pk)
        self.get(delete_url, user=structure_user, status=200)
        # ... sauf si on ne peut plus créer d'évènement
        structure_user.groups.remove(structure_group)
        structure_user.groups.add(
            GroupFactory(
                permissions__codenames=['access_admin', 'view_evenement']
            )
        )
        assertRedirectsToHome(self.get(edit_url, user=structure_user))
        assertRedirectsToHome(self.get(delete_url, user=structure_user))


class TestConsigneAdminView(ViewSetMixin):
    factory = ConsigneFactory
    model = Consigne
    namespace = 'admin_consigne'

    def test_index_filters(self, structure_user):
        ConsigneFactory.create_batch(5)
        ConsigneFactory(added_by=structure_user)

        html = self.get(self.index_url, status=200).html
        assert html.find('h2', text="Filtrer")
        assert html.select_one('select#id_added_by')
        assert html.select_one('input#id_is_available')

        html = self.get(self.index_url, user=structure_user, status=200).html
        assert not html.find('h2', text="Filtrer")

    def test_create_edit(self):
        response = self.get(self.create_url)
        assert "Ajouter une consigne" in response
        form = response.forms[1]
        assert form.action == self.create_url

        # on crée une instance
        form['consigne'] = "Une consigne avec\ndes retours à la ligne"
        assert "Consigne #1" in form.submit(status=302).follow()

        instance = self.model.objects.get(pk=1)
        assert instance.consigne == "Une consigne avec des retours à la ligne"
        assert instance.added_by == self.user

        # on peut modifier l'instance
        self.get(self.get_edit_url(instance.pk), status=200)

    def test_reset(self):
        consigne = ConsigneFactory(is_available=False)
        edit_url = self.get_edit_url(consigne.pk)

        form = self.get(edit_url).forms[1]
        reset_url = form.get('reset').attrs['formaction']
        assert reset_url == reverse('admin_consigne:reset', args=[consigne.pk])

        # on réinitialise la disponibilité
        response = form.response.goto(
            reset_url,
            params=form.submit_fields(name='reset'),
            method='post',
        )
        # ... on est redirigé vers la page de modification
        assertRedirects(response, edit_url, fetch_redirect_response=False)
        # ... un message est affiché
        messages = list(response.follow(status=200).context['messages'])
        assert len(messages) == 1
        assert messages[0].level == SUCCESS_LEVEL
        # ... et la disponibilité est modifiée
        consigne.refresh_from_db()
        assert consigne.is_available is True


patch_notify = patch(
    'danse.notifications.models.send_notification', return_value=True
)


class TestJeuSettingsAdminView(AdminViewMixin):
    url = reverse('wagtailsettings:edit', args=('jouer', 'jeusettings', 1))
    notification_title = settings.NOTIFICATIONS_JEU_TITLE

    @patch_notify
    def test_activate_deactivate(self, mock_notify, subscription):
        response = self.get(status=200)
        assert "Modification de Paramètres du jeu" in response
        form = response.forms[1]
        assert not form['is_active'].checked

        ConsigneFactory.create_batch(2, is_available=False)

        # on tente d'activer sans instruction
        form['is_active'] = True
        form['instructions'] = "null"
        response = form.submit(status=200)
        assert set(response.context['form'].errors.keys()) == {'instructions'}
        # ... et sans consigne disponible
        form['instructions'] = (
            response.context['form']
            .fields['instructions']
            .widget.format_value(
                "<p>Des <a href=\"http://example.org\">instructions</a>.</p>"
            )
        )
        response = form.submit(status=200)
        assert set(response.context['form'].errors.keys()) == {NON_FIELD_ERRORS}
        assert "Ajouter une consigne" in response

        # on réessaye avec une consigne créée
        consigne = ConsigneFactory()
        response = response.forms[1].submit(status=302).follow()
        # ... le jeu est désormais activé
        jeu_settings = JeuSettings.for_request(response.context['request'])
        assert jeu_settings.is_active is True
        # FIXME https://github.com/wagtail/wagtail/issues/7341
        assert (
            re.fullmatch(
                (
                    r'<p data-block-key=".+">Des <a href="http://example.org">'
                    'instructions</a>.</p>'
                ),
                jeu_settings.instructions,
            )
            is not None
        )
        # ... l'unique consigne est sélectionnée automatiquement
        assert jeu_settings.consigne == consigne
        assert jeu_settings.consigne.is_available is False
        assert jeu_settings.consigne.last_active_at is not None
        # ... et une notification est envoyée
        assert mock_notify.call_args.args[1] == json.dumps(
            {'title': self.notification_title, 'body': consigne.consigne}
        )

        # on réinitialise la disponibilité de la consigne
        jeu_settings.consigne.is_available = True
        jeu_settings.consigne.save()
        # ... et on ré-envoie le formulaire
        response = response.forms[1].submit(status=302).follow()
        # ... la consigne est de nouveau plus disponible
        jeu_settings = JeuSettings.for_request(response.context['request'])
        assert jeu_settings.consigne == consigne
        assert jeu_settings.consigne.is_available is False

        # on désactive enfin le jeu
        form = response.forms[1]
        form['is_active'] = False
        response = form.submit(status=302).follow()
        jeu_settings = JeuSettings.for_request(response.context['request'])
        assert jeu_settings.is_active is False

        # une seule notification a été envoyée, à l'activation
        assert mock_notify.call_count == 1

    def test_reset_consignes(self):
        ConsigneFactory.create_batch(2)
        ConsigneFactory.create_batch(9, is_available=False)

        # on active le jeu
        jeu_settings = JeuSettings.for_request(self.get().context['request'])
        jeu_settings.is_active = True
        jeu_settings.consigne = ConsigneFactory()
        jeu_settings.instructions = "<p>Des instructions</p>"
        jeu_settings.save()

        form = self.get(status=200).forms[1]
        reset_url = form.get('reset_consignes').attrs['formaction']
        assert reset_url == reverse('admin_consigne:reset_all')

        # on réinitialise la disponibilité
        response = form.response.goto(
            reset_url,
            params=form.submit_fields(name='reset_consignes'),
            method='post',
        )
        # ... on est redirigé vers la page de modification
        assertRedirects(response, self.url, fetch_redirect_response=False)
        # ... un message est affiché
        messages = list(response.follow(status=200).context['messages'])
        assert len(messages) == 1
        assert messages[0].level == SUCCESS_LEVEL
        assert "10 consigne" in messages[0].message
        # ... et la disponibilité est modifiée
        assert not Consigne.objects.filter(is_available=False).count()

    @patch_notify
    def test_change_consigne(self, mock_notify, subscription):
        consigne = ConsigneFactory()

        # on active le jeu
        jeu_settings = JeuSettings.for_request(self.get().context['request'])
        jeu_settings.is_active = True
        jeu_settings.consigne = ConsigneFactory()
        jeu_settings.instructions = "<p>Des instructions</p>"
        jeu_settings.save()
        # ... une notification est envoyée
        assert mock_notify.call_count == 1

        form = self.get(status=200).forms[1]
        change_url = form.get('change_consigne').attrs['formaction']
        assert change_url == reverse('admin_consigne:jeu_change')

        # on change de consigne
        response = form.response.goto(
            change_url,
            params=form.submit_fields(name='change_consigne'),
            method='post',
        )
        # ... on est redirigé vers la page de modification
        assertRedirects(response, self.url, fetch_redirect_response=False)
        # ... un message est affiché
        response = response.follow(status=200)
        messages = list(response.context['messages'])
        assert len(messages) == 1
        assert messages[0].level == SUCCESS_LEVEL
        # ... la consigne actuelle est modifiée
        jeu_settings = JeuSettings.for_request(response.context['request'])
        assert jeu_settings.consigne == consigne
        assert jeu_settings.consigne.is_available is False
        # ... et une notification est envoyée
        assert mock_notify.call_count == 2
        assert mock_notify.call_args.args[1] == json.dumps(
            {'title': self.notification_title, 'body': consigne.consigne}
        )

        # on tente à nouveau de changer de consigne
        response = form.response.goto(
            change_url,
            params=form.submit_fields(name='change_consigne'),
            method='post',
        ).follow()
        # ... un message d'erreur est affiché
        messages = list(response.context['messages'])
        assert len(messages) == 1
        assert messages[0].level == ERROR_LEVEL
        assert "Aucune consigne" in messages[0].message
        # ... et la consigne actuelle n'est pas modifiée
        jeu_settings = JeuSettings.for_request(response.context['request'])
        assert jeu_settings.consigne == consigne

        # une notification a été envoyée à l'activation et au changement
        assert mock_notify.call_count == 2

    def test_reset_change_unauthorized(
        self, django_app_factory, structure_user
    ):
        self.django_app = django_app_factory(csrf_checks=False)

        assertRedirectsToHome(
            self.post(
                reverse('admin_consigne:reset', args=[ConsigneFactory().pk]),
                user=structure_user,
            )
        )
        assertRedirectsToHome(
            self.get(reverse('admin_consigne:jeu_change'), user=structure_user)
        )


class TestUserAdmin(AdminViewMixin):
    def test_group_permissions(self):
        html = self.get(reverse('wagtailusers_groups:add'), status=200).html
        table = html.select_one('table.listing')
        assert table.find('h3', text="Consigne")
        assert table.find('h3', text="Évènement")
        assert table.find('h3', text="Festival")
        assert table.find('h3', text="Lieu")
        assert table.find('h3', text="Structure")
        assert table.find('h3', text="Ville")
