import operator
from functools import reduce

from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.db.models import ProtectedError, Q
from django.urls import reverse
from django.utils.text import capfirst

from wagtail.admin.admin_url_finder import AdminURLFinder
from wagtail.admin.edit_handlers import ObjectList
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.views import generic
from wagtail.admin.viewsets.model import ModelViewSet

from generic_chooser.views import ModelChooserMixin, ModelChooserViewSet


def search_queryset(queryset, search_fields, search_term):
    """Perform a database search on the given queryset."""
    lookups = [
        '{}__icontains'.format(str(search_field))
        for search_field in search_fields
    ]
    for bit in search_term.split():
        or_queries = [Q(**{lookup: bit}) for lookup in lookups]
        queryset = queryset.filter(reduce(operator.or_, or_queries))
    return queryset


# GENERIC VIEWS
# ------------------------------------------------------------------------------


class PermissionInstanceCheckedMixin:
    """
    Check that user has permission to perform action name `permission_required`
    for the given instance only. You should ensure that the class-based view on
    which you apply this mixin will call `get_object` before rendering for the
    permission to be checked.
    """

    def dispatch(self, request, *args, **kwargs):
        self.permission_instance_required = getattr(
            self, 'permission_required', None
        )
        self.permission_required = None

        return super().dispatch(request, *args, **kwargs)

    def get_object(self, queryset=None):
        obj = super().get_object(queryset)

        if self.permission_instance_required is not None:
            if not self.permission_policy.user_has_permission_for_instance(
                self.request.user, self.permission_instance_required, obj
            ):
                raise PermissionDenied

        return obj


class EditHandlerFormViewMixin:
    #: The edit handler object to use for the model.
    edit_handler = None

    def get_instance(self):
        return getattr(self, 'object') or self.model()

    def get_edit_handler(self):
        return self.edit_handler.bind_to(
            instance=self.get_instance(), request=self.request
        )

    def get_form(self):
        edit_handler = self.get_edit_handler()
        form = super().get_form(edit_handler.get_form_class())
        # Save the new edit handler object which is now bound
        self.edit_handler = edit_handler.bind_to(form=form)
        return form

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['edit_handler'] = self.edit_handler
        return context


class IndexView(generic.IndexView):
    template_name = 'admin/index.html'

    #: An optional WagtailFilterSet subclass to filter results.
    filterset_class = None

    #: An optional list or tuple of field names on which the search will apply.
    search_fields = None

    def get_queryset(self):
        queryset = super().get_queryset()
        queryset = self.filter_queryset(queryset)
        queryset = self.search_queryset(queryset)
        return queryset

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['filters'] = self.filters
        context['search_form'] = self.search_form
        context['search_url'] = (
            self.index_url_name if self.search_form else None
        )
        return context

    def get_filterset_class(self):
        """
        Return the WagtailFilterSet derived class to use for filters - if any.
        """
        return self.filterset_class

    def get_filters(self, queryset):
        """
        Return the WagtailFilterSet instance to use - if one is defined by the
        view - for the given queryset.
        """
        filterset_class = self.get_filterset_class()
        if filterset_class is None:
            return None
        return filterset_class(
            self.request.GET, queryset=queryset, request=self.request
        )

    def filter_queryset(self, queryset):
        """Filter as needed the given queryset with the filters."""
        self.filters = self.get_filters(queryset)
        if self.filters is None:
            return queryset
        return self.filters.qs

    def get_search_form(self):
        """Return the search form instance to use - if any."""
        if not self.search_fields:
            return None
        if self.request.GET.get('q', None):
            return SearchForm(self.request.GET)
        return SearchForm()

    def search_queryset(self, queryset):
        """Filter as needed the given queryset with the search term."""
        self.search_form = self.get_search_form()
        if self.search_form is None or not self.search_form.is_valid():
            return queryset
        return search_queryset(
            queryset, self.search_fields, self.search_form.cleaned_data['q']
        )


class CreateView(EditHandlerFormViewMixin, generic.CreateView):
    template_name = 'admin/create.html'


class EditView(EditHandlerFormViewMixin, generic.EditView):
    template_name = 'admin/edit.html'

    def get_delete_url(self):
        if self.delete_url_name:
            return reverse(self.delete_url_name, args=(self.object.id,))


class DeleteView(generic.DeleteView):
    template_name = 'admin/confirm_delete.html'

    #: An optional error message to display when there is protected relation.
    error_message = None

    def delete(self, request, *args, **kwargs):
        try:
            return super().delete(request, *args, **kwargs)
        except ProtectedError as e:
            context = self.get_context_data(
                protected_objects=self.format_protected_objects(
                    e.protected_objects
                )
            )
            return self.render_to_response(context)

    def format_protected_objects(self, objects):
        """
        Generate a list of 2-tuple with the string describing the object and
        its admin edit URL from the one which have a protected relation to the
        item to delete.
        """
        url_finder = AdminURLFinder(self.request.user)

        return [
            (
                '{model} : {object}'.format(
                    model=capfirst(obj.__class__._meta.verbose_name), object=obj
                ),
                url_finder.get_edit_url(obj),
            )
            for obj in objects
        ]


class ViewSet(ModelViewSet):
    index_view_class = IndexView
    add_view_class = CreateView
    edit_view_class = EditView
    delete_view_class = DeleteView

    @property
    def add_view(self):
        return self.add_view_class.as_view(
            model=self.model,
            permission_policy=self.permission_policy,
            edit_handler=self.get_edit_handler(),
            index_url_name=self.get_url_name('index'),
            add_url_name=self.get_url_name('add'),
            edit_url_name=self.get_url_name('edit'),
            header_icon=self.icon,
        )

    @property
    def edit_view(self):
        return self.edit_view_class.as_view(
            model=self.model,
            permission_policy=self.permission_policy,
            edit_handler=self.get_edit_handler(for_update=True),
            index_url_name=self.get_url_name('index'),
            edit_url_name=self.get_url_name('edit'),
            delete_url_name=self.get_url_name('delete'),
            header_icon=self.icon,
        )

    def get_edit_handler(self, for_update=False):
        """
        Return the appropriate edit_handler object to use in create and edit
        views. It must be defined either in edit_handler or panels property.
        """
        if hasattr(self, 'edit_handler'):
            return self.edit_handler
        if hasattr(self, 'panels'):
            return ObjectList(self.panels)

        raise ImproperlyConfigured(
            "Subclasses of BaseViewSet must specify 'edit_handler' or 'panels'."
        )


# CHOOSER
# ------------------------------------------------------------------------------


class ChooserMixin(ModelChooserMixin):
    #: An optional list or tuple of field names on which the search will apply.
    search_fields = None

    @property
    def is_searchable(self):
        return self.search_fields is not None

    def get_object_list(self, search_term=None, **kwargs):
        object_list = self.get_unfiltered_object_list()

        if search_term:
            object_list = search_queryset(
                object_list, self.search_fields, search_term
            )

        return object_list


class ChooserViewSet(ModelChooserViewSet):
    chooser_mixin_class = ChooserMixin

    def get_choose_view_attrs(self):
        attrs = super().get_choose_view_attrs()

        if hasattr(self, 'search_fields'):
            attrs['search_fields'] = self.search_fields

        return attrs
