from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.urls import path, reverse
from django.views.generic import View

from wagtail.admin import messages
from wagtail.admin.ui.tables import Column, TitleColumn
from wagtail.admin.views.generic.permissions import PermissionCheckedMixin
from wagtail.contrib.settings.permissions import user_can_edit_setting_type
from wagtail.core.models import Site

from danse.jouer.models import Consigne, JeuSettings

from ..filters import ConsigneFilterSet
from ..panels import consigne_panels
from ..permissions import consigne_permission_policy
from . import base


def get_jeu_settings_url(request):
    site = Site.find_for_request(request)

    return reverse(
        'wagtailsettings:edit',
        kwargs={
            'app_name': 'jouer',
            'model_name': 'jeusettings',
            'site_pk': getattr(site, 'pk', None),
        },
    )


class ConsigneIndexView(base.IndexView):
    add_item_label = "Ajouter une consigne"
    page_title = "Consignes"
    paginate_by = 20

    columns = [
        Column('id', label="#", sort_key='id'),
        TitleColumn(
            'consigne', url_name='admin_consigne:edit', label="Consigne"
        ),
        Column('added_by', label="Ajoutée par", sort_key='added_by'),
    ]

    def get_filterset_class(self):
        if self.permission_policy.user_has_permission(
            self.request.user, 'change'
        ):
            return ConsigneFilterSet
        return None


class ConsigneCreateView(base.CreateView):
    error_message = "La consigne ne peut être enregistrée du fait d'erreurs."
    page_title = "Ajouter une consigne"
    success_message = "Consigne '{0}' ajoutée."

    def save_instance(self):
        self.form.instance.added_by = self.request.user
        return super().save_instance()


class ConsigneEditView(base.EditView):
    delete_item_label = "Supprimer la consigne"
    error_message = "La consigne ne peut être enregistrée du fait d'erreurs."
    success_message = "Consigne '{0}' mise à jour."
    template_name = 'admin/consigne/edit.html'


class ConsigneDeleteView(base.DeleteView):
    confirmation_message = (
        "Êtes-vous sûr⋅e de vouloir supprimer cette consigne ?"
    )
    page_title = "Supprimer la consigne"
    success_message = "Consigne '{0}' supprimée."


class ConsigneResetView(PermissionCheckedMixin, View):
    permission_required = 'edit'

    def post(self, request, pk):
        consigne = get_object_or_404(Consigne, pk=pk)
        consigne.is_available = True
        consigne.save(update_fields=['is_available'])

        messages.success(
            request, "La disponibilité de la consigne a été réinitialisée."
        )

        return HttpResponseRedirect(reverse('admin_consigne:edit', args=[pk]))


class ConsigneResetAllView(ConsigneResetView):
    def post(self, request):
        nb_updated = Consigne.objects.filter(is_available=False).update(
            is_available=True
        )

        messages.success(
            request,
            "La disponibilité de %d consigne(s) a été réinitialisée."
            % (nb_updated),
        )

        return HttpResponseRedirect(get_jeu_settings_url(request))


class JeuConsigneChangeView(View):
    def dispatch(self, request, *args, **kwargs):
        if not user_can_edit_setting_type(request.user, JeuSettings):
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)

    def post(self, request):
        jeu_settings = JeuSettings.for_request(request)

        try:
            jeu_settings.change_consigne()
        except Consigne.DoesNotExist:
            messages.error(
                request, "Aucune consigne n'est disponible pour le jeu."
            )
        else:
            messages.success(request, "La consigne du jeu a été changée.")

        return HttpResponseRedirect(get_jeu_settings_url(request))


class ConsigneViewSet(base.ViewSet):
    icon = 'form'
    model = Consigne
    permission_policy = consigne_permission_policy

    index_view_class = ConsigneIndexView
    add_view_class = ConsigneCreateView
    edit_view_class = ConsigneEditView
    delete_view_class = ConsigneDeleteView

    panels = consigne_panels

    def get_urlpatterns(self):
        return super().get_urlpatterns() + [
            path(
                'reset/',
                ConsigneResetAllView.as_view(
                    permission_policy=self.permission_policy
                ),
                name='reset_all',
            ),
            path(
                '<int:pk>/reset/',
                ConsigneResetView.as_view(
                    permission_policy=self.permission_policy
                ),
                name='reset',
            ),
            path(
                'change/',
                JeuConsigneChangeView.as_view(),
                name='jeu_change',
            ),
        ]
