from wagtail.admin.edit_handlers import ObjectList
from wagtail.admin.ui.tables import Column, TitleColumn

from danse.core.models import Evenement

from ..filters import EvenementAdminFilterSet, EvenementFilterSet
from ..forms import EvenementForm
from ..panels import evenement_panels
from ..permissions import evenement_permission_policy
from . import base


class IndexView(base.IndexView):
    add_item_label = "Ajouter un évènement"
    page_title = "Évènements"
    paginate_by = 20
    search_fields = ('nom',)

    def get_queryset(self):
        queryset = super().get_queryset().select_related('lieu', 'structure')
        if self.request.user.is_structure_user:
            queryset = queryset.filter(structure=self.request.user.structure_id)
        return queryset

    def get_columns(self):
        columns = [
            TitleColumn(
                'nom',
                url_name='admin_evenement:edit',
                label="Nom",
                sort_key='nom',
            ),
            Column('debut', label="Début", sort_key='debut'),
            Column('lieu', label="Lieu"),
        ]
        if not self.request.user.is_structure_user:
            columns.insert(2, Column('structure', label="Structure"))
        return columns

    def get_filterset_class(self):
        if not self.request.user.is_structure_user:
            return EvenementAdminFilterSet
        return EvenementFilterSet


class EvenementFormMixin:
    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['can_choose_structure'] = not self.request.user.is_structure_user
        return kwargs


class CreateView(EvenementFormMixin, base.CreateView):
    error_message = "L'évènement ne peut être enregistré du fait d'erreurs."
    page_title = "Ajouter un évènement"
    success_message = "Évènement '{0}' ajouté."

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        if self.request.user.is_structure_user:
            kwargs['initial'] = {'lieu': self.request.user.structure.lieu}
        return kwargs

    def save_instance(self):
        if self.request.user.is_structure_user:
            self.form.instance.structure = self.request.user.structure
        return super().save_instance()


class EditView(
    EvenementFormMixin, base.PermissionInstanceCheckedMixin, base.EditView
):
    delete_item_label = "Supprimer l'évènement"
    error_message = "L'évènement ne peut être enregistré du fait d'erreurs."
    success_message = "Évènement '{0}' mis à jour."


class DeleteView(base.PermissionInstanceCheckedMixin, base.DeleteView):
    confirmation_message = (
        "Êtes-vous sûr⋅e de vouloir supprimer cet évènement ?"
    )
    page_title = "Supprimer l'évènement"
    success_message = "Évènement '{0}' supprimé."


class EvenementViewSet(base.ViewSet):
    icon = 'date'
    model = Evenement
    permission_policy = evenement_permission_policy

    index_view_class = IndexView
    add_view_class = CreateView
    edit_view_class = EditView
    delete_view_class = DeleteView

    edit_handler = ObjectList(evenement_panels, base_form_class=EvenementForm)
