from django.core.serializers import serialize
from django.http import Http404
from django.urls import path
from django.utils.functional import cached_property

from wagtail.api.v2.filters import OrderingFilter
from wagtail.api.v2.utils import BadRequestError
from wagtail.api.v2.views import BaseAPIViewSet

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import exceptions
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.settings import api_settings
from rest_framework.viewsets import GenericViewSet

from danse.utils.api.filters import SearchFilter
from danse.utils.api.viewsets import DjangoFilterViewSetMixin
from danse.utils.http import ICalResponse

from .. import models
from .filters import EvenementFilter
from .serializers import (
    EvenementSerializer,
    FestivalSerializer,
    LieuSerializer,
    StructureSerializer,
    VilleSerializer,
)


class GeoJSONRenderer(JSONRenderer):
    media_type = 'application/geo+json'
    format = 'geojson'


class CoreAPIViewSet(DjangoFilterViewSetMixin, BaseAPIViewSet):
    renderer_classes = api_settings.DEFAULT_RENDERER_CLASSES

    #: Liste des filtres à utiliser pour la vue en liste
    listing_filter_backends = [
        DjangoFilterBackend,
        OrderingFilter,
        SearchFilter,
    ]

    #: Message d'erreur à retourner lorsque l'élément est introuvable
    not_found_error = "L'élément demandé est introuvable."

    @cached_property
    def filter_backends(self):
        """Retourne les filtres à utiliser en fonction de l'action."""
        if self.action == 'listing_view':
            return self.listing_filter_backends
        return []

    def handle_exception(self, exc):
        """
        Garde un comportement uniforme dans le traitement des erreurs en
        utilisant uniquement celui de Django REST Framework.
        """
        if isinstance(exc, Http404):
            exc = exceptions.NotFound(self.not_found_error)
        elif isinstance(exc, BadRequestError):
            exc = exceptions.ParseError(
                "La requête est incorrecte ({}).".format(exc)
            )
        return GenericViewSet.handle_exception(self, exc)

    def initial(self, *args, **kwargs):
        super().initial(*args, **kwargs)
        # stocke le format de la réponse demandé
        self.format = self.request.GET.get('format', None)


class GeoJSONAPIViewSet(CoreAPIViewSet):
    #: Le nom du champ définissant la position géographique.
    geo_field = 'geom'

    #: Liste des champs à ajouter aux propriétés du point.
    geo_properties_fields = []

    renderer_classes = CoreAPIViewSet.renderer_classes + [GeoJSONRenderer]

    def listing_view(self, request):
        """
        Répond avec la liste des éléments sans pagination et correctement
        sérialisée lorsque le format GeoJSON est demandé.
        """
        if self.format == 'geojson':
            queryset = self.get_queryset().exclude(
                **{'{}__isnull'.format(self.geo_field): True}
            )
            self.check_query_parameters(queryset)
            queryset = self.filter_queryset(queryset)
            serialized_data = self.get_geojson_serialized_data(queryset)
            return Response(serialized_data)
        return super().listing_view(request)

    def detail_view(self, request, pk):
        """
        Répond avec l'élément correctement sérialisé lorsque le format
        GeoJSON est demandé.
        """
        if self.format == 'geojson':
            instance = self.get_object()
            serialized_data = self.get_geojson_serialized_data(
                [instance], as_collection=False
            )
            return Response(serialized_data)
        return super().detail_view(request, pk)

    def get_geojson_serialized_data(self, queryset, as_collection=True):
        """Sérialise le ou les éléments au format GeoJSON."""
        return serialize(
            'geojson',
            queryset,
            geometry_field=self.geo_field,
            fields=self.geo_properties_fields,
            as_collection=as_collection,
            to_json=False,
        )


# TERRITOIRE
# ------------------------------------------------------------------------------


class VillesAPIViewSet(GeoJSONAPIViewSet):
    base_serializer_class = VilleSerializer
    listing_default_fields = BaseAPIViewSet.listing_default_fields + [
        'nom',
        'code_postal',
        'departement',
        'visuel',
        'url_wikipedia',
    ]
    nested_default_fields = BaseAPIViewSet.nested_default_fields + [
        'nom',
        'code_postal',
        'departement',
        'visuel',
    ]
    geo_properties_fields = ['nom', 'code_postal']
    name = 'villes'
    model = models.Ville

    not_found_error = "La ville demandée est introuvable."


class LieuxAPIViewSet(GeoJSONAPIViewSet):
    base_serializer_class = LieuSerializer
    listing_default_fields = BaseAPIViewSet.listing_default_fields + [
        'nom',
        'rue',
        'ville',
        'coordinates',
        'structure_id',
    ]
    nested_default_fields = BaseAPIViewSet.nested_default_fields + [
        'nom',
        'rue',
        'ville',
        'coordinates',
        'structure_id',
    ]
    geo_properties_fields = ['nom']
    name = 'lieux'
    model = models.Lieu

    not_found_error = "Le lieu demandé est introuvable."

    def get_queryset(self):
        return super().get_queryset().select_related('ville', 'structure')


class StructuresAPIViewSet(CoreAPIViewSet):
    base_serializer_class = StructureSerializer
    listing_default_fields = BaseAPIViewSet.listing_default_fields + [
        'nom',
        'accroche',
        'visuel',
        'email',
        'telephone',
        'url_site',
    ]
    nested_default_fields = BaseAPIViewSet.nested_default_fields + [
        'nom',
        'accroche',
        'visuel',
    ]
    name = 'structures'
    model = models.Structure

    not_found_error = "La structure demandée est introuvable."

    def get_queryset(self):
        return super().get_queryset().select_related('lieu', 'lieu__ville')


# EVENEMENT
# ------------------------------------------------------------------------------


class FestivalsAPIViewSet(CoreAPIViewSet):
    base_serializer_class = FestivalSerializer
    listing_default_fields = BaseAPIViewSet.listing_default_fields + [
        'nom',
        'visuel',
        'debut',
        'fin',
    ]
    nested_default_fields = BaseAPIViewSet.nested_default_fields + [
        'nom',
        'visuel',
        'debut',
        'fin',
    ]
    name = 'festivals'
    model = models.Festival

    not_found_error = "Le festival demandé est introuvable."


class EvenementsAPIViewSet(CoreAPIViewSet):
    base_serializer_class = EvenementSerializer
    meta_fields = BaseAPIViewSet.meta_fields + ['ical_url']
    listing_default_fields = BaseAPIViewSet.listing_default_fields + [
        'nom',
        'genre',
        'visuel',
        'artiste',
        'structure',
        'lieu',
        'debut',
        'duree',
    ]
    filterset_class = EvenementFilter
    nested_default_fields = BaseAPIViewSet.nested_default_fields + [
        'nom',
        'genre',
        'visuel',
        'artiste',
        'structure',
        'lieu',
        'debut',
        'duree',
    ]
    name = 'evenements'
    model = models.Evenement

    not_found_error = "L'évènement demandé est introuvable."

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .order_by('debut')
            .select_related('structure', 'lieu', 'lieu__ville', 'festival')
        )

    @classmethod
    def get_urlpatterns(cls):
        return super().get_urlpatterns() + [
            path(
                '<int:pk>/ical/',
                cls.as_view({'get': 'detail_ical_view'}),
                name='detail_ical',
            ),
        ]

    def detail_ical_view(self, request, pk):
        instance = self.get_object()
        event = instance.as_ical_event()
        return ICalResponse(events=(event,))
