from django.contrib.gis.db import models
from django.core.exceptions import ValidationError

from wagtail.core.fields import RichTextField
from wagtail.search import index

from dynamic_filenames import FilePattern
from ics import Event
from stdimage.models import StdImageField

from danse.utils.html import html_to_text

from .common import VISUEL_HELP_TEXT, VISUEL_VALIDATORS, VISUEL_VARIATIONS


class Festival(models.Model):
    """
    Un festival sur une période donnée, constitué d'évènements.
    """

    class Meta:
        ordering = ('debut', 'nom')
        verbose_name = "festival"
        verbose_name_plural = "festivals"

    visuel_upload_to = FilePattern(
        filename_pattern='festivals/visuel/{uuid:x}{ext}'
    )

    # Champs

    nom = models.CharField("nom", max_length=60)
    description = RichTextField("description")
    visuel = StdImageField(
        "visuel",
        upload_to=visuel_upload_to,
        variations=dict(
            VISUEL_VARIATIONS,
            thumbnail={'width': 200, 'height': 200, 'crop': True},
        ),
        delete_orphans=True,
        help_text=VISUEL_HELP_TEXT,
        validators=VISUEL_VALIDATORS,
    )
    url_externe = models.URLField("lien externe", blank=True)

    debut = models.DateField("début")
    fin = models.DateField("fin")

    # Utilisation des champs

    api_fields = [
        'nom',
        'description',
        'visuel',
        'url_externe',
        'debut',
        'fin',
        'evenements',
    ]

    search_fields = [
        index.AutocompleteField('nom'),
        index.SearchField('nom', partial_match=True),
    ]

    # Méthodes

    def __str__(self):
        return self.nom

    def clean(self):
        if (
            self.debut is not None
            and self.fin is not None
            and self.debut >= self.fin
        ):
            raise ValidationError(
                "La date de fin doit être après celle de début.",
                code='end_before_start',
            )


class Evenement(index.Indexed, models.Model):
    """
    Un évènement à une date et heure donnée, organisé par une ou plusieurs
    structures.
    """

    class Meta:
        ordering = ('debut', 'nom')
        verbose_name = "évènement"
        verbose_name_plural = "évènements"

    class Genre(models.TextChoices):
        SPECTACLE = 'spectacle'
        ATELIER = 'atelier'

    visuel_upload_to = FilePattern(
        filename_pattern='evenements/visuel/{uuid:x}{ext}'
    )

    # Champs

    nom = models.CharField("nom", max_length=60)
    genre = models.CharField("genre", max_length=10, choices=Genre.choices)
    artiste = models.CharField("artiste", max_length=100)
    description = RichTextField("description")
    visuel = StdImageField(
        "visuel",
        upload_to=visuel_upload_to,
        variations=VISUEL_VARIATIONS,
        delete_orphans=True,
        help_text=VISUEL_HELP_TEXT,
        validators=VISUEL_VALIDATORS,
    )
    url_externe = models.URLField("lien externe", blank=True)

    lieu = models.ForeignKey(
        'core.Lieu',
        on_delete=models.PROTECT,
        related_name='evenements',
        verbose_name="lieu",
    )
    structure = models.ForeignKey(
        'core.Structure',
        on_delete=models.PROTECT,
        related_name='evenements',
        verbose_name="structure",
    )

    debut = models.DateTimeField("début")
    duree = models.DurationField("durée", blank=True, null=True)

    age_min = models.PositiveSmallIntegerField(
        "âge minimum",
        blank=True,
        null=True,
        help_text="Laissez vide si l'évènement est pour tout public.",
    )

    tarif_plein = models.DecimalField(
        "tarif plein",
        max_digits=5,
        decimal_places=2,
        blank=True,
        null=True,
        help_text="Laissez vide si l'évènement est gratuit.",
    )
    tarif_reduit = models.DecimalField(
        "tarif réduit",
        max_digits=5,
        decimal_places=2,
        blank=True,
        null=True,
        help_text=(
            "Saisissez le plus petit tarif réduit s'il y en a plusieurs de "
            "disponibles."
        ),
    )
    url_billetterie = models.URLField("lien vers la billetterie", blank=True)

    festival = models.ForeignKey(
        Festival,
        on_delete=models.SET_NULL,
        related_name='evenements',
        blank=True,
        null=True,
        verbose_name="festival",
    )

    date_created = models.DateTimeField(auto_now_add=True)
    date_updated = models.DateTimeField(auto_now=True)

    # Utilisation des champs

    api_fields = [
        'nom',
        'genre',
        'description',
        'visuel',
        'url_externe',
        'artiste',
        'structure',
        'lieu',
        'debut',
        'duree',
        'age_min',
        'tarif_plein',
        'tarif_reduit',
        'url_billetterie',
        'festival',
    ]

    search_fields = [
        index.AutocompleteField('nom'),
        index.SearchField('nom', partial_match=True),
        index.SearchField('artiste', partial_match=True),
    ]

    # Propriétés

    @property
    def date(self):
        return self.debut.date() if self.debut else None

    @property
    def heure(self):
        return self.debut.time() if self.debut else None

    # Méthodes

    def __str__(self):
        return self.nom

    def clean(self):
        errors = {}

        if (
            self.tarif_plein
            and self.tarif_reduit is not None
            and self.tarif_reduit >= self.tarif_plein
        ):
            errors['tarif_reduit'] = ValidationError(
                "Ce montant doit être inférieur au tarif plein.",
                code='invalid',
            )

        if (
            self.date is not None
            and self.festival is not None
            and (
                self.date > self.festival.fin or self.date < self.festival.debut
            )
        ):
            errors['festival'] = ValidationError(
                "Les dates du festival sont en dehors de cet évènement.",
                code='invalid',
            )

        if errors:
            raise ValidationError(errors)

    def save(self, *args, **kwargs):
        # normalise les valeurs nulles
        if not self.age_min:
            self.age_min = None
        if not self.tarif_plein:
            self.tarif_plein = None
            self.tarif_reduit = None
        elif not self.tarif_reduit:
            self.tarif_reduit = None

        super().save(*args, **kwargs)

    def as_ical_event(self):
        return Event(
            name=self.nom,
            description=html_to_text(self.description),
            location=self.lieu.adresse,
            begin=self.debut,
            duration=self.duree,
            created=self.date_created,
            last_modified=self.date_updated,
        )
