from django.contrib.gis.db import models
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator

from wagtail.core.fields import RichTextField
from wagtail.search import index

from dynamic_filenames import FilePattern
from phonenumber_field.modelfields import PhoneNumberField
from stdimage.models import StdImageField

from danse.utils.postcode import get_departement
from danse.utils.wikipedia import WIKIPEDIA_PAGE_URL_RE

from .common import VISUEL_HELP_TEXT, VISUEL_VALIDATORS, VISUEL_VARIATIONS

validate_wikipedia_page_url = RegexValidator(
    WIKIPEDIA_PAGE_URL_RE,
    message="Entrez une URL d'une page Wikipedia valide.",
    code='invalid',
)


class GeomMixin:
    """
    Ajoute des facilités pour les propriétés du champ géométrique.
    """

    @property
    def coordinates(self):
        if self.geom is None:
            return None
        return {
            'lon': self.geom.x,
            'lat': self.geom.y,
        }


class Ville(GeomMixin, index.Indexed, models.Model):
    """
    Une ville de la région, avec quelques informations pratiques.
    """

    class Meta:
        constraints = [
            models.UniqueConstraint(
                fields=['nom', 'code_postal'], name='unique_ville'
            )
        ]
        ordering = ('nom',)
        verbose_name = "ville"
        verbose_name_plural = "villes"

    visuel_upload_to = FilePattern(
        filename_pattern='villes/visuel/{uuid:x}{ext}'
    )

    # Champs

    nom = models.CharField("ville", max_length=100)
    code_postal = models.CharField(
        "code postal",
        max_length=5,
        validators=[RegexValidator(r'\d{5}')],
    )
    geom = models.PointField("géolocalisation", null=True)

    description = RichTextField("description")
    visuel = StdImageField(
        "visuel",
        upload_to=visuel_upload_to,
        variations=dict(
            VISUEL_VARIATIONS,
            thumbnail={'width': 200, 'height': 200, 'crop': True},
        ),
        delete_orphans=True,
        help_text=VISUEL_HELP_TEXT,
        validators=VISUEL_VALIDATORS,
    )
    url_wikipedia = models.URLField(
        "lien vers Wikipedia",
        blank=True,
        validators=[validate_wikipedia_page_url],
    )

    @property
    def departement(self):
        return get_departement(self.code_postal)

    # Utilisation des champs

    api_fields = [
        'nom',
        'code_postal',
        'departement',
        'coordinates',
        'description',
        'visuel',
        'url_wikipedia',
    ]

    search_fields = [
        index.AutocompleteField('nom'),
        index.SearchField('nom', partial_match=True, boost=10),
        index.SearchField('code_postal', partial_match=True),
    ]

    # Méthodes

    def __str__(self):
        return self.nom

    def get_code_postal_display(self):
        return "{} {}".format(self.code_postal[:2], self.code_postal[2:])

    def unique_error_message(self, *args, **kwargs):
        return ValidationError(
            message="Une ville avec ce nom et ce code postal existe déjà.",
            code='unique_together',
        )


class Lieu(GeomMixin, index.Indexed, models.Model):
    """
    Un lieu dans une ville, dans lequel peut se passer des évènements et où
    peut siéger une structure.
    """

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=['nom', 'ville'], name='unique_lieu')
        ]
        ordering = ('nom',)
        verbose_name = "lieu"
        verbose_name_plural = "lieux"

    # Champs

    nom = models.CharField("nom", max_length=60)

    rue = models.CharField("rue", max_length=100)
    ville = models.ForeignKey(
        Ville,
        on_delete=models.PROTECT,
        related_name='lieux',
        verbose_name="ville",
    )
    geom = models.PointField("géolocalisation", null=True)

    @property
    def adresse(self):
        return '{}\n{}\n{} {}'.format(
            self.nom,
            self.rue,
            self.ville.get_code_postal_display(),
            self.ville.nom,
        )

    # Utilisation des champs

    api_fields = [
        'nom',
        'rue',
        'ville',
        'coordinates',
        'structure_id',
    ]

    search_fields = [
        index.AutocompleteField('nom'),
        index.SearchField('nom', partial_match=True),
    ]

    # Méthodes

    def __str__(self):
        if not hasattr(self, 'ville'):
            return self.nom
        return '{} ({})'.format(self.nom, self.ville)


class Structure(index.Indexed, models.Model):
    """
    Une structure de la région, qui organise des évènements.
    """

    class Meta:
        ordering = ('nom',)
        verbose_name = "structure"
        verbose_name_plural = "structures"

    visuel_upload_to = FilePattern(
        filename_pattern='structures/visuel/{uuid:x}{ext}'
    )

    # Champs

    nom = models.CharField("nom", max_length=60, unique=True)
    accroche = models.CharField(
        "accroche",
        max_length=80,
        help_text=(
            "Une courte description ou une phrase d'accroche qui apparaîtra "
            "en dessous du nom."
        ),
    )
    description = RichTextField("description")
    visuel = StdImageField(
        "visuel",
        upload_to=visuel_upload_to,
        variations=dict(
            VISUEL_VARIATIONS,
            thumbnail={'width': 200, 'height': 200, 'crop': True},
        ),
        delete_orphans=True,
        help_text=VISUEL_HELP_TEXT,
        validators=VISUEL_VALIDATORS,
    )

    email = models.EmailField("adresse courriel", blank=True)
    telephone = PhoneNumberField("téléphone", blank=True)
    url_site = models.URLField("lien du site", blank=True)
    url_billetterie = models.URLField("lien vers la billetterie", blank=True)

    lieu = models.OneToOneField(
        Lieu,
        on_delete=models.PROTECT,
        related_name='structure',
        blank=True,
        null=True,
        verbose_name="lieu",
    )

    # Utilisation des champs

    api_fields = [
        'nom',
        'accroche',
        'description',
        'visuel',
        'email',
        'telephone',
        'url_site',
        'url_billetterie',
        'lieu',
    ]

    search_fields = [
        index.AutocompleteField('nom'),
        index.SearchField('nom', partial_match=True),
    ]

    # Méthodes

    def __str__(self):
        return self.nom
