import datetime

from django.test.client import Client
from django.urls import reverse
from django.utils.timezone import now

from .factories import (
    EvenementFactory,
    FestivalFactory,
    LieuFactory,
    StructureFactory,
    VilleFactory,
)


class APIViewSet:
    name = None

    @classmethod
    def setup_class(cls):
        cls.client = Client()

    def get_listing_response(self, data=None):
        return self.client.get(
            reverse('api:{}:listing'.format(self.name)), data
        )

    def get_detail_response(self, item_id, data=None):
        return self.client.get(
            reverse('api:{}:detail'.format(self.name), args=(item_id,)), data
        )


class TestCoreViewSet(APIViewSet):
    name = 'villes'

    def test_not_found(self):
        response = self.get_detail_response(100)
        assert response.status_code == 404
        assert response.json() == {
            'detail': "La ville demandée est introuvable."
        }

    def test_bad_request(self):
        response = self.get_listing_response({'order': "truc"})
        assert response.status_code == 400
        assert response.json() == {
            'detail': (
                "La requête est incorrecte (cannot order by 'truc' "
                "(unknown field))."
            )
        }


class TestVillesViewSet(APIViewSet):
    name = 'villes'

    def test_listing(self):
        VilleFactory.create_batch(2)

        response = self.get_listing_response()
        assert response.status_code == 200

        content = response.json()
        assert content['meta'] == {'total_count': 2}
        assert len(content['items']) == 2

        for item in content['items']:
            assert set(item.keys()) == {
                'id',
                'meta',
                'nom',
                'code_postal',
                'departement',
                'visuel',
                'url_wikipedia',
            }
            assert set(item['meta'].keys()) == {'type', 'detail_url'}

    def test_listing_search(self, libreville):
        # recherche sur nom et code_postal
        response = self.get_listing_response({'search': "libr"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 1
        response = self.get_listing_response({'search': "01"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 1

        # recherche sur un champ non référencé
        response = self.get_listing_response({'search': "wikipedia"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 0

    def test_listing_as_geojson(self):
        VilleFactory.create_batch(2)
        VilleFactory(geom=None)

        response = self.get_listing_response({'format': 'geojson'})
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/geo+json'

        content = response.json()
        assert content['type'] == 'FeatureCollection'
        assert len(content['features']) == 2
        assert set(content['features'][0]['properties'].keys()) == {
            'nom',
            'code_postal',
        }

    def test_detail(self, libreville):
        response = self.get_detail_response(libreville.id)
        assert response.status_code == 200
        assert response.json() == {
            'id': 1,
            'meta': {
                'type': 'core.Ville',
                'detail_url': 'http://localhost/api/v1/villes/1/',
            },
            'nom': "Libreville",
            'code_postal': "01234",
            'departement': "Ain",
            'coordinates': {'lon': 2.8, 'lat': 50.68},
            'description': '<p>Une description de la ville.</p>',
            'visuel': {
                'url': 'http://testserver{}'.format(libreville.visuel.url),
                'variations': {
                    'mobile': 'http://testserver{}'.format(
                        libreville.visuel.mobile.url
                    ),
                    'desktop': 'http://testserver{}'.format(
                        libreville.visuel.desktop.url
                    ),
                    'thumbnail': 'http://testserver{}'.format(
                        libreville.visuel.thumbnail.url
                    ),
                },
            },
            'url_wikipedia': "https://fr.wikipedia.org/wiki/Libreville",
        }

    def test_detail_as_geojson(self, libreville):
        response = self.get_detail_response(
            libreville.id, {'format': 'geojson'}
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/geo+json'
        assert response.json() == {
            'id': '1',
            'type': 'Feature',
            'geometry': {'type': 'Point', 'coordinates': [2.8, 50.68]},
            'properties': {'nom': "Libreville", 'code_postal': "01234"},
        }


class TestLieuxViewSet(APIViewSet):
    name = 'lieux'

    def test_listing(self):
        LieuFactory.create_batch(2)

        response = self.get_listing_response()
        assert response.status_code == 200

        content = response.json()
        assert content['meta'] == {'total_count': 2}
        assert len(content['items']) == 2

        for item in content['items']:
            assert set(item.keys()) == {
                'id',
                'meta',
                'nom',
                'rue',
                'ville',
                'coordinates',
                'structure_id',
            }
            assert set(item['meta'].keys()) == {'type', 'detail_url'}
            assert set(item['ville'].keys()) == {
                'id',
                'meta',
                'nom',
                'code_postal',
                'departement',
                'visuel',
            }

    def test_listing_search(self, olieux):
        # recherche sur nom
        response = self.get_listing_response({'search': "oli"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 1

        # recherche sur un champ non référencé
        response = self.get_listing_response({'search': "marguerite"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 0

    def test_listing_as_geojson(self):
        LieuFactory.create_batch(2)
        LieuFactory(geom=None)

        response = self.get_listing_response({'format': 'geojson'})
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/geo+json'

        content = response.json()
        assert content['type'] == 'FeatureCollection'
        assert len(content['features']) == 2
        assert set(content['features'][0]['properties'].keys()) == {'nom'}

    def test_detail(self, olieux, libreville):
        response = self.get_detail_response(olieux.id)
        assert response.status_code == 200
        assert response.json() == {
            'id': 1,
            'meta': {
                'type': 'core.Lieu',
                'detail_url': 'http://localhost/api/v1/lieux/1/',
            },
            'nom': "Olieux",
            'rue': "13 rue des marguerites",
            'ville': {
                'id': 1,
                'meta': {
                    'type': 'core.Ville',
                    'detail_url': 'http://localhost/api/v1/villes/1/',
                },
                'nom': "Libreville",
                'code_postal': "01234",
                'departement': "Ain",
                'visuel': {
                    'url': 'http://testserver{}'.format(libreville.visuel.url),
                    'variations': {
                        'mobile': 'http://testserver{}'.format(
                            libreville.visuel.mobile.url
                        ),
                        'desktop': 'http://testserver{}'.format(
                            libreville.visuel.desktop.url
                        ),
                        'thumbnail': 'http://testserver{}'.format(
                            libreville.visuel.thumbnail.url
                        ),
                    },
                },
            },
            'coordinates': {'lon': 2.9, 'lat': 50.68},
            'structure_id': None,
        }

    def test_detail_as_geojson(self, olieux):
        response = self.get_detail_response(olieux.id, {'format': 'geojson'})
        assert response.status_code == 200
        assert response['Content-Type'] == 'application/geo+json'
        assert response.json() == {
            'id': '1',
            'type': 'Feature',
            'geometry': {'coordinates': [2.9, 50.68], 'type': 'Point'},
            'properties': {'nom': "Olieux"},
        }


class TestStructuresViewSet(APIViewSet):
    name = 'structures'

    def test_listing(self):
        StructureFactory.create_batch(2)

        response = self.get_listing_response()
        assert response.status_code == 200

        content = response.json()
        assert content['meta'] == {'total_count': 2}
        assert len(content['items']) == 2

        for item in content['items']:
            assert set(item.keys()) == {
                'id',
                'meta',
                'nom',
                'accroche',
                'visuel',
                'email',
                'telephone',
                'url_site',
            }
            assert set(item['meta'].keys()) == {'type', 'detail_url'}
            assert set(item['visuel'].keys()) == {'url', 'variations'}

    def test_listing_search(self, destructure):
        # recherche sur nom
        response = self.get_listing_response({'search': "déstru"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 1

        # recherche sur un champ non référencé
        response = self.get_listing_response({'search': "description"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 0

    def test_detail(self, destructure, libreville):
        response = self.get_detail_response(destructure.id)
        assert response.status_code == 200
        assert response.json() == {
            'id': 1,
            'meta': {
                'type': 'core.Structure',
                'detail_url': 'http://localhost/api/v1/structures/1/',
            },
            'nom': "Déstructure",
            'accroche': "Une scène de la région",
            'description': "<p>Une description de <b>Déstructure</b>.</p>",
            'visuel': {
                'url': 'http://testserver{}'.format(destructure.visuel.url),
                'variations': {
                    'mobile': 'http://testserver{}'.format(
                        destructure.visuel.mobile.url
                    ),
                    'desktop': 'http://testserver{}'.format(
                        destructure.visuel.desktop.url
                    ),
                    'thumbnail': 'http://testserver{}'.format(
                        destructure.visuel.thumbnail.url
                    ),
                },
            },
            'email': "contact@destructure.fr",
            'telephone': {
                'national': "09 87 65 43 21",
                'rfc3966': "tel:+33-9-87-65-43-21",
            },
            'url_site': "https://destructure.fr",
            'url_billetterie': "https://destructure.fr/billetterie/",
            'lieu': {
                'id': 1,
                'meta': {
                    'type': 'core.Lieu',
                    'detail_url': 'http://localhost/api/v1/lieux/1/',
                },
                'nom': "Olieux",
                'rue': "13 rue des marguerites",
                'ville': {
                    'id': 1,
                    'meta': {
                        'type': 'core.Ville',
                        'detail_url': 'http://localhost/api/v1/villes/1/',
                    },
                    'nom': "Libreville",
                    'code_postal': "01234",
                    'departement': "Ain",
                    'visuel': {
                        'url': 'http://testserver{}'.format(
                            libreville.visuel.url
                        ),
                        'variations': {
                            'mobile': 'http://testserver{}'.format(
                                libreville.visuel.mobile.url
                            ),
                            'desktop': 'http://testserver{}'.format(
                                libreville.visuel.desktop.url
                            ),
                            'thumbnail': 'http://testserver{}'.format(
                                libreville.visuel.thumbnail.url
                            ),
                        },
                    },
                },
                'coordinates': {'lon': 2.9, 'lat': 50.68},
                'structure_id': destructure.id,
            },
        }

    def test_detail_without_telephone(self):
        response = self.get_detail_response(StructureFactory(telephone='').id)
        assert response.status_code == 200
        assert response.json()['telephone'] is None

    def test_detail_without_lieu(self):
        response = self.get_detail_response(StructureFactory(lieu=None).id)
        assert response.status_code == 200
        assert response.json()['lieu'] is None


class TestFestivalsViewSet(APIViewSet):
    name = 'festivals'

    def test_listing(self):
        FestivalFactory.create_batch(2)

        response = self.get_listing_response()
        assert response.status_code == 200

        content = response.json()
        assert content['meta'] == {'total_count': 2}
        assert len(content['items']) == 2

        for item in content['items']:
            assert set(item.keys()) == {
                'id',
                'meta',
                'nom',
                'visuel',
                'debut',
                'fin',
            }
            assert set(item['meta'].keys()) == {'type', 'detail_url'}
            assert set(item['visuel'].keys()) == {'url', 'variations'}

    def test_detail(self, festoch):
        response = self.get_detail_response(festoch.id)
        assert response.status_code == 200
        assert response.json() == {
            'id': 1,
            'meta': {
                'type': 'core.Festival',
                'detail_url': 'http://localhost/api/v1/festivals/1/',
            },
            'nom': "Festoch'",
            'description': "<p>Une description du <b>Festoch'</b> !</p>",
            'visuel': {
                'url': 'http://testserver{}'.format(festoch.visuel.url),
                'variations': {
                    'mobile': 'http://testserver{}'.format(
                        festoch.visuel.mobile.url
                    ),
                    'desktop': 'http://testserver{}'.format(
                        festoch.visuel.desktop.url
                    ),
                    'thumbnail': 'http://testserver{}'.format(
                        festoch.visuel.thumbnail.url
                    ),
                },
            },
            'url_externe': "https://festoch.org",
            'debut': '2020-01-17',
            'fin': '2020-01-19',
            'evenements': [],
        }

    def test_detail_with_evenements(self, festoch):
        EvenementFactory.create_batch(2, festival=festoch)

        response = self.get_detail_response(festoch.id)
        assert response.status_code == 200

        content = response.json()
        assert len(content['evenements']) == 2

        for item in content['evenements']:
            assert set(item.keys()) == {
                'id',
                'meta',
                'nom',
                'genre',
                'visuel',
                'artiste',
                'structure',
                'lieu',
                'debut',
                'duree',
            }
            assert set(item['lieu'].keys()) == {
                'id',
                'meta',
                'nom',
                'rue',
                'ville',
                'coordinates',
                'structure_id',
            }


class TestEvenementsViewSet(APIViewSet):
    name = 'evenements'

    def test_listing(self):
        EvenementFactory.create_batch(
            2, debut=now() + datetime.timedelta(days=2)
        )

        response = self.get_listing_response()
        assert response.status_code == 200

        content = response.json()
        assert content['meta'] == {'total_count': 2}
        assert len(content['items']) == 2

        for item in content['items']:
            assert set(item.keys()) == {
                'id',
                'meta',
                'nom',
                'genre',
                'visuel',
                'artiste',
                'structure',
                'lieu',
                'debut',
                'duree',
            }
            assert set(item['meta'].keys()) == {'type', 'detail_url'}
            assert set(item['visuel'].keys()) == {'url', 'variations'}
            assert set(item['structure'].keys()) == {
                'id',
                'meta',
                'nom',
                'accroche',
                'visuel',
            }
            assert set(item['lieu'].keys()) == {
                'id',
                'meta',
                'nom',
                'rue',
                'ville',
                'coordinates',
                'structure_id',
            }

    def test_listing_filters(self):
        EvenementFactory(debut_str='2020-01-17 18:00')
        EvenementFactory(debut_str='2020-02-21 14:00')
        EvenementFactory(debut_str='2020-02-21 18:00')

        # tous les évènements sont dans le passé
        response = self.get_listing_response()
        assert response.status_code == 200
        assert len(response.json()['items']) == 0

        # filtre à une date précise
        response = self.get_listing_response({'debut': '2020-02-21'})
        assert response.status_code == 200
        assert len(response.json()['items']) == 2

        # filtre avant une date
        response = self.get_listing_response({'debut__lte': '2020-01-17'})
        assert response.status_code == 200
        assert len(response.json()['items']) == 1

        # filtre après une date
        response = self.get_listing_response({'debut__gte': '2020-02-21'})
        assert response.status_code == 200
        assert len(response.json()['items']) == 2

    def test_listing_filters_invalid(self):
        response = self.get_listing_response({'debut__lt': '2020-02-21'})
        assert response.status_code == 400
        assert set(response.json().keys()) == {'detail'}

        response = self.get_listing_response({'debut': 'blabla'})
        assert response.status_code == 400
        assert set(response.json().keys()) == {'debut'}

    def test_listing_search(self):
        EvenementFactory(nom="ÈveDanse")
        EvenementFactory(nom="Autre titre", artiste="Ève")
        EvenementFactory(nom="Autre titre aussi")

        # recherche sur le titre et l'artiste indépendamment du début
        response = self.get_listing_response({'search': "Ève"})
        assert response.status_code == 200
        assert len(response.json()['items']) == 2

    def test_detail(self, evedanse, destructure, libreville):
        response = self.get_detail_response(evedanse.id)
        assert response.status_code == 200
        assert response.json() == {
            'id': 1,
            'meta': {
                'type': 'core.Evenement',
                'detail_url': 'http://localhost/api/v1/evenements/1/',
                'ical_url': 'http://localhost/api/v1/evenements/1/ical/',
            },
            'nom': "ÈveDanse",
            'genre': "spectacle",
            'description': "<p>Une description de <b>ÈveDanse</b> !</p>",
            'visuel': {
                'url': 'http://testserver{}'.format(evedanse.visuel.url),
                'variations': {
                    'mobile': 'http://testserver{}'.format(
                        evedanse.visuel.mobile.url
                    ),
                    'desktop': 'http://testserver{}'.format(
                        evedanse.visuel.desktop.url
                    ),
                },
            },
            'url_externe': "https://olieux.fr/evedanse/",
            'artiste': "Ève - Cie Vivante",
            'structure': {
                'id': 1,
                'meta': {
                    'type': 'core.Structure',
                    'detail_url': 'http://localhost/api/v1/structures/1/',
                },
                'nom': "Déstructure",
                'accroche': "Une scène de la région",
                'visuel': {
                    'url': 'http://testserver{}'.format(destructure.visuel.url),
                    'variations': {
                        'mobile': 'http://testserver{}'.format(
                            destructure.visuel.mobile.url
                        ),
                        'desktop': 'http://testserver{}'.format(
                            destructure.visuel.desktop.url
                        ),
                        'thumbnail': 'http://testserver{}'.format(
                            destructure.visuel.thumbnail.url
                        ),
                    },
                },
            },
            'lieu': {
                'id': 1,
                'meta': {
                    'type': 'core.Lieu',
                    'detail_url': 'http://localhost/api/v1/lieux/1/',
                },
                'nom': "Olieux",
                'rue': "13 rue des marguerites",
                'ville': {
                    'id': 1,
                    'meta': {
                        'type': 'core.Ville',
                        'detail_url': 'http://localhost/api/v1/villes/1/',
                    },
                    'nom': "Libreville",
                    'code_postal': "01234",
                    'departement': "Ain",
                    'visuel': {
                        'url': 'http://testserver{}'.format(
                            libreville.visuel.url
                        ),
                        'variations': {
                            'mobile': 'http://testserver{}'.format(
                                libreville.visuel.mobile.url
                            ),
                            'desktop': 'http://testserver{}'.format(
                                libreville.visuel.desktop.url
                            ),
                            'thumbnail': 'http://testserver{}'.format(
                                libreville.visuel.thumbnail.url
                            ),
                        },
                    },
                },
                'coordinates': {'lon': 2.9, 'lat': 50.68},
                'structure_id': destructure.id,
            },
            'debut': evedanse.debut.isoformat(),
            'duree': 5400,
            'age_min': None,
            'tarif_plein': '10.00',
            'tarif_reduit': '3.00',
            'url_billetterie': "https://olieux.fr/billeterie/",
            'festival': None,
        }

    def test_detail_without_duree(self):
        response = self.get_detail_response(EvenementFactory(duree=None).id)
        assert response.status_code == 200
        assert response.json()['duree'] is None

    def test_detail_with_festival(self, evedanse, festoch):
        evedanse.festival = festoch
        evedanse.save()

        response = self.get_detail_response(evedanse.id)
        assert response.status_code == 200

        content = response.json()
        assert set(content['festival'].keys()) == {
            'id',
            'meta',
            'nom',
            'visuel',
            'debut',
            'fin',
        }

    def test_detail_ical(self, evedanse):
        response = self.client.get(
            reverse('api:{}:detail_ical'.format(self.name), args=(evedanse.id,))
        )
        assert response.status_code == 200
        assert response['Content-Type'] == 'text/calendar'
        assert "SUMMARY:ÈveDanse\r\n".encode('utf-8') in list(response)
