import datetime

from django.core.exceptions import NON_FIELD_ERRORS, ValidationError
from django.db.models import ProtectedError
from django.utils.dateparse import parse_datetime

import pytest

from ..models import Evenement, Festival, Lieu, Structure, Ville
from .factories import (
    EvenementFactory,
    FestivalFactory,
    LieuFactory,
    StructureFactory,
    VilleFactory,
)


class TestVille:
    def test_str(self):
        assert str(Ville(nom="Libreville")) == "Libreville"

    def test_code_postal(self):
        assert Ville(code_postal="01234").get_code_postal_display() == "01 234"

    def test_coordinates(self, libreville):
        assert libreville.coordinates == {
            'lon': 2.8,
            'lat': 50.68,
        }
        assert Ville(geom=None).coordinates is None

    def test_clean(self, libreville):
        VilleFactory.build(
            nom="Autreville", code_postal=libreville.code_postal
        ).full_clean()

        with pytest.raises(ValidationError) as excinfo:
            Ville(
                nom=libreville.nom, code_postal=libreville.code_postal
            ).full_clean()
        unique_error = excinfo.value.error_dict[NON_FIELD_ERRORS][0]
        assert unique_error.code == 'unique_together'
        assert "ce nom et ce code postal" in str(unique_error)

        with pytest.raises(ValidationError) as excinfo:
            Ville(nom="Autreville", code_postal="1234").full_clean()
        assert excinfo.value.error_dict['code_postal'][0].code == 'invalid'


class TestLieu:
    def test_str(self, libreville):
        lieu = Lieu(nom="Olieux")
        assert str(lieu) == "Olieux"

        lieu.ville = libreville
        assert str(lieu) == "Olieux (Libreville)"

    def test_clean(self, olieux):
        with pytest.raises(ValidationError) as excinfo:
            LieuFactory.build(nom=olieux.nom, ville=olieux.ville).full_clean()
        assert excinfo.value.error_dict[NON_FIELD_ERRORS][0].code == (
            'unique_together'
        )

    def test_adresse(self, olieux):
        assert (
            olieux.adresse
            == "Olieux\n13 rue des marguerites\n01 234 Libreville"
        )


class TestStructure:
    def test_str(self):
        assert str(Structure(nom="Le Vivat")) == "Le Vivat"

    def test_clean(self, olieux):
        structure = StructureFactory(lieu=None)
        structure.full_clean()

        structure.lieu = olieux
        structure.full_clean()
        structure.save()

        with pytest.raises(ProtectedError):
            olieux.delete()


class TestFestival:
    def test_str(self):
        assert str(Festival(nom="Festival #1")) == "Festival #1"

    def test_clean(self):
        festival = FestivalFactory()
        festival.full_clean()

    @pytest.mark.parametrize(
        'debut, fin',
        [('2020-01-01', '2020-01-01'), ('2020-01-02', '2020-01-01')],
    )
    def test_clean_end_before_start(self, debut, fin):
        with pytest.raises(ValidationError) as excinfo:
            FestivalFactory(debut_str=debut, fin_str=fin).full_clean()
        assert excinfo.value.error_dict[NON_FIELD_ERRORS][0].code == (
            'end_before_start'
        )


class TestEvenement:
    def test_str(self):
        assert str(Evenement(nom="Danse #1")) == "Danse #1"

    def test_date_heure(self):
        evenement = Evenement()
        assert evenement.date is None
        assert evenement.heure is None

        evenement.debut = parse_datetime('2020-01-01 18:00')
        assert evenement.date == datetime.date(2020, 1, 1)
        assert evenement.heure == datetime.time(18)

    def test_clean(self):
        evenement = EvenementFactory()
        evenement.full_clean()

    def test_clean_tarif(self):
        evenement = EvenementFactory.build(tarif_plein=10)

        # tarif réduit == tarif plein
        with pytest.raises(ValidationError) as excinfo:
            evenement.tarif_reduit = evenement.tarif_plein
            evenement.clean()
        assert 'tarif_reduit' in excinfo.value.error_dict

        # tarif réduit > tarif plein
        with pytest.raises(ValidationError) as excinfo:
            evenement.tarif_reduit = 12
            evenement.clean()
        assert 'tarif_reduit' in excinfo.value.error_dict

        # tarif plein > tarif réduit
        evenement.tarif_plein = 20
        evenement.clean()
        assert evenement.tarif_reduit == 12

    def test_sanitize_empty_values(self):
        evenement = EvenementFactory(tarif_plein=0, tarif_reduit=5, age_min=10)
        # évènement gratuit
        assert evenement.tarif_plein is None
        assert evenement.tarif_reduit is None
        # ... et avec un âge minimum
        assert evenement.age_min == 10

        # évènement sans tarif réduit
        evenement.tarif_plein = 10
        evenement.tarif_reduit = 0
        evenement.save()
        assert evenement.tarif_plein == 10
        assert evenement.tarif_reduit is None

        # évènement sans âge minimum
        evenement.age_min = 0
        evenement.save()
        assert evenement.age_min is None

    def test_clean_festival(self):
        festival = FestivalFactory(debut_str='2020-01-17', fin_str='2020-01-19')

        for debut in ('2020-01-17 18:00', '2020-01-19 18:00'):
            EvenementFactory.build(debut_str=debut, festival=festival).clean()

        for debut in ('2020-01-16 18:00', '2020-01-20 18:00'):
            with pytest.raises(ValidationError) as excinfo:
                EvenementFactory.build(
                    debut_str=debut, festival=festival
                ).clean()
            assert 'festival' in excinfo.value.error_dict
