from django.test.client import Client
from django.urls import reverse

from wagtail.core.models import Page
from wagtail.tests.utils.form_data import rich_text

import pytest

from danse.utils.tests import ViewMixin

from ..models import StandardPage
from .factories import StandardPageFactory


@pytest.fixture
def root_page():
    return Page.objects.filter(sites_rooted_here__is_default_site=True).get()


class TestStandardPage(ViewMixin):
    model = StandardPage

    @classmethod
    def setup_class(cls):
        cls.client = Client()

    def test_admin_add(self, admin_user, root_page):
        url = reverse(
            'wagtailadmin_pages:add',
            args=(
                self.model._meta.app_label,
                self.model._meta.model_name,
                root_page.pk,
            ),
        )

        form = self.get(url, user=admin_user).forms[1]
        form['title'] = "À propos"
        form['body'] = rich_text("<p>Une description de <b>À propos</b>.</p>")
        form['slug'] = "a-propos"
        form.submit(status=302).follow()

        page = self.model.objects.get(title="À propos")
        assert page.show_in_menus is False

    def test_api_listing(self, root_page):
        StandardPageFactory(parent=root_page)
        StandardPageFactory.create_batch(
            4, parent=root_page, show_in_menus=True
        )
        # crée d'autres pages en dehors du site
        StandardPageFactory.create_batch(2)

        url = reverse('api:pages:listing')

        response = self.client.get(url)
        assert response.status_code == 200
        content = response.json()
        assert content['meta'] == {'total_count': 6}  # = root_page + 5
        assert len(content['items']) == 6
        assert set(content['items'][0].keys()) == {'id', 'meta', 'title'}

        # filtre avec les pages à afficher dans le menu
        response = self.client.get(url, {'show_in_menus': '1'})
        assert response.status_code == 200
        assert len(response.json()['items']) == 4

    def test_api_detail(self, root_page):
        page = StandardPageFactory(
            parent=root_page,
            title="Une page",
            body='<p>avec un <a linktype="page" id="{}">lien</a>.</p>'.format(
                StandardPageFactory(parent=root_page, slug='test').id
            ),
        )
        url = reverse('api:pages:detail', args=(page.id,))

        response = self.client.get(url)
        assert response.status_code == 200
        content = response.json()
        assert set(content.keys()) == {'id', 'meta', 'title', 'body'}
        assert content['title'] == "Une page"
        assert content['body'] == (
            '<p>avec un <a href="/page/test/">lien</a>.</p>'
        )
