import random

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.text import normalize_newlines

from wagtail.admin.edit_handlers import FieldPanel, HelpPanel, MultiFieldPanel
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.core.fields import RichTextField


class ConsigneQuerySet(models.QuerySet):
    def available(self):
        """Filtre la sélection avec les consignes disponibles."""
        return self.filter(is_available=True)

    def get_random(self):
        """Retourne une consigne aléatoirement parmi la sélection."""
        available_ids = list(self.values_list('id', flat=True))
        if not available_ids:
            raise self.model.DoesNotExist()
        if len(available_ids) == 1:
            return self.first()
        return self.get(pk=random.choice(available_ids))


class Consigne(models.Model):
    """
    Une consigne d'un jeu, qui apparaîtra aléatoirement un jour de l'année.
    """

    class Meta:
        ordering = ('id',)
        verbose_name = "consigne"
        verbose_name_plural = "consignes"

    # Champs

    consigne = models.TextField("consigne", max_length=200)
    is_available = models.BooleanField("disponible", default=True)

    added_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        null=True,
        editable=False,
        verbose_name="ajouté par",
    )
    last_active_at = models.DateField(
        "active dernièrement le", null=True, blank=True, editable=False
    )

    objects = ConsigneQuerySet.as_manager()

    # Méthodes

    def __str__(self):
        return "Consigne #{}".format(self.id)

    def save(self, *args, **kwargs):
        self.consigne = normalize_newlines(self.consigne).replace('\n', ' ')
        super().save(*args, **kwargs)


@register_setting(icon='puzzle-piece')
class JeuSettings(BaseSetting):
    """
    Paramètres du jeu au sein de l'application.
    """

    class Meta:
        verbose_name = "jouer"

    # Champs

    is_active = models.BooleanField(
        "actif",
        default=False,
        help_text=(
            "Détermine si le jeu est disponible au sein de l'application."
        ),
    )
    consigne = models.ForeignKey(
        Consigne,
        limit_choices_to={'is_available': True},
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name="consigne actuelle",
    )
    instructions = RichTextField(
        features=['bold', 'italic', 'link'],
        blank=True,
        verbose_name="instructions",
    )

    select_related = ['consigne']

    # Administration de Wagtail

    panels = [
        MultiFieldPanel(
            [
                FieldPanel('is_active'),
                HelpPanel(
                    template='admin/edit_handlers/jeu_consigne_panel.html'
                ),
            ],
            heading="Statut",
        ),
        FieldPanel('instructions'),
    ]

    # Méthodes

    def __str__(self):
        return "Paramètres du jeu"

    def clean(self):
        if not self.is_active:
            return

        if not self.instructions:
            raise ValidationError(
                {'instructions': "Veuillez renseigner comment participer."}
            )

        if not self.consigne:
            try:
                self.change_consigne(False)
            except Consigne.DoesNotExist:
                raise ValidationError(
                    "Aucune consigne n'est disponible pour le jeu."
                )

    def change_consigne(self, commit=True):
        self.consigne = Consigne.objects.available().get_random()

        if commit:
            self.save(update_fields=['consigne'])
