import json
import logging

from django.db import models

from .utils import send_notification

logger = logging.getLogger('danse.notifications')


class PushSubscriptionManager(models.Manager):
    def update_or_create(self, subscription_info):
        """
        Crée ou mets à jour l'abonnement avec le même point d'entrée, et
        retourne un tuple (subscription, changed).
        """
        try:
            endpoint = subscription_info['endpoint']
            keys = subscription_info['keys']
        except (KeyError, TypeError, ValueError) as exc:
            logger.error(
                "Informations d'abonnement %r incorrectes (%s)",
                subscription_info,
                exc,
            )
            raise ValueError("Les informations d'abonnement sont incorrectes")
        subscription, changed = self.get_or_create(
            endpoint=endpoint, defaults={'keys': keys}
        )
        if not changed and subscription.keys != keys:
            subscription.keys = keys
            subscription.save()
            changed = True
        return subscription, changed


class PushSubscriptionQuerySet(models.QuerySet):
    def notify(self, title, body):
        """
        Envoie une notification à tous les abonnements de la sélection, et
        retourne un tuple (nb_succès, nb_erreurs).
        """
        errors = 0
        success = 0
        data = json.dumps({'title': title, 'body': body})
        for subscription in self:
            if subscription.notify(data):
                success += 1
            else:
                errors += 1
        return (success, errors)


class PushSubscription(models.Model):
    """
    Un abonnement aux notifications via l'API Push.
    """

    class Meta:
        ordering = ('-date_updated',)

    endpoint = models.URLField(max_length=512, primary_key=True)
    keys = models.JSONField()
    date_created = models.DateTimeField(auto_now_add=True)
    date_updated = models.DateTimeField(auto_now=True)

    objects = PushSubscriptionManager.from_queryset(PushSubscriptionQuerySet)()

    @property
    def info(self):
        return {
            'endpoint': self.endpoint,
            'keys': self.keys,
        }

    def notify(self, data):
        return send_notification(self, data)

    def __repr__(self):
        return f'<PushSubscription {self.endpoint}>'
