from django.test import override_settings
from django.urls import reverse

import pytest
from rest_framework.test import APIClient

from ..models import PushSubscription


class TestNotificationsAPIViewSet:
    info_url = reverse('api:notifications:info')
    subscribe_url = reverse('api:notifications:subscribe')
    unsubscribe_url = reverse('api:notifications:unsubscribe')

    @classmethod
    def setup_class(cls):
        cls.client = APIClient(enforce_csrf_checks=True)

    def subscribe(self, data, format='json'):
        return self.client.post(self.subscribe_url, data, format=format)

    def unsubscribe(self, data, format='json'):
        return self.client.post(self.unsubscribe_url, data, format=format)

    @override_settings(VAPID_SERVER_KEY="abcd")
    def test_info(self):
        response = self.client.get(self.info_url)
        assert response.status_code == 200
        assert response.json() == {'server_key': "abcd"}

    def test_subscribe_json_expected(self):
        response = self.subscribe({}, format=None)
        assert response.status_code == 415
        assert "Unsupported media type" in response.json()['detail']

    def test_subscribe_info_required(self):
        response = self.subscribe({})
        assert response.status_code == 400
        assert response.json() == {
            'detail': "Le paramètre 'subscription' est requis.",
        }

    @pytest.mark.parametrize(
        'subscription_info',
        (None, ['endpoint'], {'endpoint': 'xx'}, {'keys': 'xx'}),
    )
    def test_subscribe_info_malformed(self, subscription_info):
        response = self.subscribe({'subscription': subscription_info})
        assert response.status_code == 400
        assert response.json() == {
            'detail': "Le paramètre 'subscription' est incorrect.",
        }

    def test_subscribe_info_invalid_url(self):
        # FIXME: la validation n'est pas faite sur 'endpoint' !
        subscription_info = {
            'endpoint': 'xx',
            'keys': {'p256dh': '0123abcde...', 'auth': 'abc123...'},
        }
        response = self.subscribe({'subscription': subscription_info})
        assert response.status_code == 200

    def test_subscribe_info_create_and_update(self):
        # on enregistre un abonnement
        subscription_info = {
            'endpoint': 'https://push.example.com/v1/12345',
            'keys': {'p256dh': '0123abcde...', 'auth': 'abc123...'},
        }
        response = self.subscribe({'subscription': subscription_info})
        assert response.status_code == 200
        assert response.json() == {
            'detail': "L'abonnement a été enregistré.",
        }

        subscription = PushSubscription.objects.first()
        assert subscription.endpoint == subscription_info['endpoint']
        assert subscription.keys == subscription_info['keys']

        # on envoie les mêmes informations
        response = self.subscribe({'subscription': subscription_info})
        assert response.status_code == 200
        assert response.json() == {
            'detail': "L'abonnement est déjà à jour.",
        }

        # ... l'abonnement n'est pas mis à jour
        assert PushSubscription.objects.all().count() == 1
        new_subscription = PushSubscription.objects.first()
        assert new_subscription.date_updated == subscription.date_updated

        # on mets à jour les clés
        subscription_info['keys']['auth'] = '123abcd'
        response = self.subscribe({'subscription': subscription_info})
        assert response.status_code == 200
        assert response.json() == {
            'detail': "L'abonnement a été enregistré.",
        }

        # ... l'abonnement est mis à jour
        new_subscription.refresh_from_db()
        assert new_subscription.keys == subscription_info['keys']
        assert new_subscription.date_updated != subscription.date_updated

    def test_unsubscribe_endpoint_required(self):
        response = self.unsubscribe({})
        assert response.status_code == 400
        assert response.json() == {
            'detail': "Le paramètre 'endpoint' est requis.",
        }

    def test_unsubscribe_endpoint_unknown(self):
        response = self.unsubscribe({'endpoint': 'abcd'})
        assert response.status_code == 404
        assert response.json() == {
            'detail': "Aucun abonnement trouvé pour ce 'endpoint'.",
        }

    def test_unsubscribe(self, subscription):
        response = self.unsubscribe({'endpoint': subscription.endpoint})
        assert response.status_code == 200
        assert response.json() == {
            'detail': "L'abonnement a été supprimé.",
        }

        with pytest.raises(PushSubscription.DoesNotExist):
            subscription.refresh_from_db()
