from unittest.mock import patch

import pytest
from pywebpush import WebPushException
from requests import Response

from ..models import PushSubscription
from .factories import PushSubscriptionFactory


class TestSendNotification:
    def test_sending(self, subscription):
        with patch('danse.notifications.utils.webpush') as mock_webpush:
            mock_webpush.return_value.status_code = 200

            assert subscription.notify("Un message") is True
            assert mock_webpush.call_args.kwargs['data'] == "Un message"
            assert mock_webpush.call_args.kwargs['vapid_claims']['sub'] == (
                'mailto:tech@cliss21.com'
            )

    def test_expired_subscription(self, subscription):
        response = Response()
        response.status_code = 410

        with patch(
            'danse.notifications.utils.webpush',
            side_effect=WebPushException("error", response=response),
        ):
            assert subscription.notify("Un message") is False

            with pytest.raises(PushSubscription.DoesNotExist):
                subscription.refresh_from_db()

    def test_error(self, subscription, caplog):
        response = Response()
        response.status_code = 500

        with patch(
            'danse.notifications.utils.webpush',
            side_effect=WebPushException("webpusherror", response=response),
        ):
            assert subscription.notify("Un message") is False
            assert "webpusherror" in caplog.text

            subscription.refresh_from_db()

    def test_notify_from_queryset(self):
        PushSubscriptionFactory.create_batch(3)

        with patch(
            'danse.notifications.models.send_notification',
            side_effect=[True, False, False],
        ) as mock_notify:
            assert PushSubscription.objects.notify("Title", "Body") == (1, 2)
            assert mock_notify.call_args.args[1] == (
                '{"title": "Title", "body": "Body"}'
            )
