/* global ol */
'use strict';

{
  const SEARCH_BASE_URL = 'https://nominatim.openstreetmap.org/search';

  const jsonFormat = new ol.format.GeoJSON();

  function toQueryString(params) {
    return Object.keys(params).map((key) => {
      return encodeURIComponent(key) + '=' + encodeURIComponent(params[key]);
    }).join('&');
  }

  /**
   * Étend un objet MapWidget afin de récupérer les coordonnées d'un
   * point en fonction de la valeur d'autres champs.
   *
   * @class
   * @param {MapWidget} widget
   * @param {Object} options
   * @param {boolean} options.readOnly - Détermine si le point ne peut pas
   * être déplacé.
   * @param {boolean} options.searchParams - Les paramètres à ajouter à l'URL
   * lors de la recherche.
   * @param {boolean} options.searchParamsMap - Les associations entre un
   * paramètre de recherche et le nom du ou des champs.
   */
  function GeocodedPoint(widget, options) {
    this.fields = [];
    this.params = {};
    this.widget = widget;

    this.options = Object.assign({
      readOnly: false,
      searchParams: {},
      searchParamsMap: {},
    }, options);

    // récupère le(s) champ(s) associé(s) à chaque paramètre
    Object.keys(this.options.searchParamsMap).forEach((param) => {
      let names = this.options.searchParamsMap[param];

      if (typeof names === 'string') {
        names = [names];
      }

      this.options.searchParamsMap[param] = names.map((name) => {
        const field = document.getElementById(`id_${name}`)
          || document.querySelector(`[name="${name}"]`);

        if (!field) {
          throw new Error(`Unable to find the field '${name}'`);
        }

        if (this.fields.indexOf(field) === -1) {
          this.fields.push(field);
        }

        return field;
      });
    });

    this.fields.forEach((field) => {
      // écoute les changements de la valeur du champ
      field.addEventListener('change', this.lookup.bind(this));
    });

    if (this.options.readOnly) {
      this.widget.interactions.draw.setActive(false);
      this.widget.map.removeInteraction(this.widget.interactions.modify);
    }
  }

  /**
   * Tente de récupérer les coordonnées du point.
   */
  GeocodedPoint.prototype.lookup = function () {
    const params = this.getSearchParams();
    // force le format de la réponse
    params.format = 'geojson';

    fetch(`${SEARCH_BASE_URL}?${toQueryString(params)}`)
      .then(response => response.json())
      .then(data => {
        const point = data.features && data.features.find(
          ({ geometry }) => geometry.type === 'Point'
        );

        if (!point) {
          if (this.options.readOnly) {
            this.widget.clearFeatures();
            this.widget.interactions.draw.setActive(false);
          }

          throw new Error('Unable to locate the address');
        }

        this.widget.featureCollection.clear();

        // crée et place le point suivant la projection de la carte
        const feature = jsonFormat.readFeature(point, {
          dataProjection: 'EPSG:4326',
          featureProjection: this.widget.map.getView().getProjection(),
        });
        this.widget.featureOverlay.getSource().addFeature(feature);
        this.widget.map.getView().fit(feature.getGeometry().getExtent(), {
          maxZoom: this.widget.options.default_zoom
        });

        // mets à jour le champ avec les nouvelles coordonnées
        this.widget.serializeFeatures();
      });
  };

  /**
   * Construis les paramètres de recherche avec la valeur des champs.
   * @returns {Object}
   */
  GeocodedPoint.prototype.getSearchParams = function() {
    const params = Object.assign({}, this.options.searchParams);

    Object.keys(this.options.searchParamsMap).forEach((param) => {
      const values = [];

      if (params[param]) {
        values.push(params[param]);
      }

      for (const field of this.options.searchParamsMap[param]) {
        let value = field.value;

        if (!value) {
          continue;
        }

        if (field.type === 'hidden') {
          const instanceSelectorWidget = document.getElementById(
            `${field.id}-instance-selector-widget`
          );
          if (!instanceSelectorWidget) {
            throw new Error(
              `Unable to find value of hidden field ${field.name}`
            );
          }

          const title = instanceSelectorWidget.querySelector(
            '.instance-selector-widget__display__title'
          );
          if (!title.textContent) {
            continue;
          }

          value = title.textContent;
        }

        values.push(value);
      }

      params[param] = values.join(', ');
    });

    return params;
  };

  window.GeocodedPoint = GeocodedPoint;
}
