from django import forms

from wagtail.users.forms import User, UserCreationForm, UserEditForm

from danse.core.models import Structure

from .models import StructureUser


class StructureUserForm(forms.Form):
    is_for_structure = forms.BooleanField(
        initial=True, required=False, label="Associer à une structure"
    )
    structure = forms.ModelChoiceField(
        queryset=Structure.objects.order_by('nom'),
        required=False,
        label="Structure",
        help_text=(
            "Laissez vide pour donner la possibilité au compte de créer sa "
            "structure."
        ),
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['first_name'].required = False

    def set_user_meta(self):
        opts = self._meta
        opts.model = User
        opts.fields = set(opts.fields) - {'structure'}

    def set_structure_user_meta(self):
        opts = self._meta
        opts.model = StructureUser
        opts.fields = set(opts.fields) | {'structure'}


class StructureUserCreationForm(StructureUserForm, UserCreationForm):
    """
    Crée si besoin un compte associé à une structure.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['password1'].help_text = (
            "Si le mot de passe n'est pas défini, ce compte utilisateur devra "
            "demander la réinitialisation de son mot de passe afin de le "
            "définir et pouvoir se connecter."
        )

    def clean(self):
        cleaned_data = super().clean()
        is_for_structure = cleaned_data.get('is_for_structure')

        if not self.errors and is_for_structure:
            self.set_structure_user_meta()
            self.instance = self._meta.model()

        return cleaned_data


class StructureUserEditForm(StructureUserForm, UserEditForm):
    """
    Mets à jour le compte, et crée ou supprime si besoin l'association avec
    une structure.
    """

    def __init__(self, *args, instance, **kwargs):
        initial = kwargs.pop('initial', {})

        try:
            structure_user = StructureUser.objects.get_for_user(instance)
        except StructureUser.DoesNotExist:
            initial['is_for_structure'] = False
        else:
            self.set_structure_user_meta()
            instance = structure_user
            initial['is_for_structure'] = True

        super().__init__(*args, instance=instance, initial=initial, **kwargs)

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        return self.instance

    def _save_m2m(self):
        is_for_structure = self.cleaned_data['is_for_structure']

        if is_for_structure != self.initial['is_for_structure']:
            if is_for_structure:
                self.set_structure_user_meta()
                self.instance = StructureUser.objects.create_for_user(
                    self.instance
                )
                self.instance.structure = self.cleaned_data['structure']
                self.instance.save()
            else:
                self.set_user_meta()
                self.instance = self.instance.user_ptr
                StructureUser.objects.delete_for_user(self.instance)

        super()._save_m2m()
