import re
from urllib.parse import urlparse

from django.contrib.auth.models import AnonymousUser

import pytest

from danse.utils.tests import WAGTAILHOME_URL, ViewMixin

from ..models import StructureUser, User
from .factories import StructureUserFactory, UserFactory

RE_URL = re.compile(r'(http(?:|s)://[\S]+/?)$', re.MULTILINE)


@pytest.fixture
def user():
    return UserFactory()


@pytest.fixture
def structure_user():
    return StructureUserFactory()


class TestStructureUserManager:
    def test_get_for_user_does_not_exist(self, user):
        with pytest.raises(StructureUser.DoesNotExist):
            StructureUser.objects.get_for_user(AnonymousUser())
        with pytest.raises(StructureUser.DoesNotExist):
            StructureUser.objects.get_for_user(user)

    def test_get_for_user(self, structure_user):
        user = User.objects.get(pk=structure_user.pk)
        assert StructureUser.objects.get_for_user(user) == structure_user

    def test_create_for_user_exists(self, structure_user):
        user = User.objects.get(pk=structure_user.pk)
        assert StructureUser.objects.create_for_user(user) == structure_user

    def test_create_for_user(self):
        user = UserFactory(first_name="Camille", last_name="Dupond")

        structure_user = StructureUser.objects.create_for_user(user)
        assert structure_user.user_ptr == user
        assert structure_user.first_name == "Camille"
        assert structure_user.last_name == "Dupond"
        assert StructureUser.objects.count() == 0

        structure_user.save()
        user.refresh_from_db()
        assert user.first_name == "Camille"
        assert user.last_name == "Dupond"
        assert StructureUser.objects.count() == 1

    def test_delete_for_user(self):
        structure_user = StructureUserFactory(last_name='Structure')
        user = User.objects.get(pk=structure_user.pk)

        assert StructureUser.objects.delete_for_user(user) == (
            1,
            {'users.StructureUser': 1},
        )
        assert User.objects.filter(pk=user.pk, last_name='Structure').exists()


class TestStructureUserWelcomeMail(ViewMixin):
    def test_send_without_password(self, mailoutbox):
        StructureUserFactory(username='user_name', last_name='Déstructure')

        assert len(mailoutbox) == 1
        body = mailoutbox[0].body
        assert "user_name" in body
        assert "*Déstructure*" in body
        assert WAGTAILHOME_URL in body

        assert "définir votre mot de passe" in body
        reset_url = urlparse(RE_URL.findall(body)[1])
        assert reset_url.netloc == 'localhost'
        self.get(reset_url.path).follow().form

    def test_send_with_password(self, mailoutbox):
        StructureUserFactory(password='hashedpass')

        assert len(mailoutbox) == 1
        assert "définir votre mot de passe" not in mailoutbox[0].body

    def test_send_created(self, mailoutbox):
        structure_user = StructureUserFactory()

        assert len(mailoutbox) == 1
        mailoutbox = []

        structure_user.last_name = "Autre nom"
        structure_user.save()

        assert len(mailoutbox) == 0
