from django.urls import reverse

from danse.core.tests.factories import StructureFactory
from danse.utils.tests import WAGTAILHOME_URL, AdminViewMixin, ViewMixin

from ..models import StructureUser, User
from .factories import StructureUserFactory, UserFactory


class TestWagtailUsersAddEditView(AdminViewMixin):
    add_url = reverse('wagtailusers_users:add')

    def get_edit_url(self, user_id):
        return reverse('wagtailusers_users:edit', args=[user_id])

    def test_add_edit_user(self):
        StructureFactory()

        form = self.get(self.add_url).forms[1]

        # on crée un compte utilisateur
        form['username'] = 'camille'
        form['email'] = 'camille@example.org'
        form['first_name'] = "Camille"
        form['last_name'] = "Dupond"
        form['is_for_structure'] = False
        form['password1'] = form['password2'] = "UnPassCompliqué"
        form.submit(status=302).follow()

        user = User.objects.get(username='camille')
        assert user.password
        assert not StructureUser.objects.filter(username='camille').exists()

        # on vérifie la modification de ce compte
        form = self.get(self.get_edit_url(user.pk), status=200).forms[1]
        assert form['is_for_structure'].checked is False
        assert not form['structure'].value

    def test_add_user_without_password(self):
        StructureFactory()

        form = self.get(self.add_url).forms[1]

        # on crée un compte utilisateur sans mot de passe
        form['username'] = 'camille'
        form['email'] = 'camille@example.org'
        form['first_name'] = "Camille"
        form['last_name'] = "Dupond"
        form['is_for_structure'] = False
        form.submit(status=302).follow()

        user = User.objects.get(username='camille')
        assert not user.password

    def test_add_edit_structure_user(self):
        structure = StructureFactory(nom="Olieux")

        form = self.get(self.add_url).forms[1]

        # on crée un compte utilisateur
        form['username'] = 'olieux'
        form['email'] = 'olieux@example.org'
        form['first_name'] = ""
        form['last_name'] = "Olieux"
        form['is_for_structure'] = True
        form['structure'].select(text="Olieux")
        form['password1'] = form['password2'] = "UnPassCompliqué"
        form.submit(status=302).follow()

        user = StructureUser.objects.get(email='olieux@example.org')
        assert user.structure == structure

        # on vérifie la modification de ce compte
        form = self.get(self.get_edit_url(user.pk), status=200).forms[1]
        assert form['is_for_structure'].checked is True
        assert form['structure'].value == str(structure.id)

        # on modifie l'association à une structure
        form['structure'] = ''
        form.submit(status=302).follow()

        user.refresh_from_db()
        assert not user.structure

    def test_edit_user_to_structure_user(self):
        structure = StructureFactory(nom="Olieux")
        user = UserFactory()
        url = self.get_edit_url(user.pk)

        form = self.get(url).forms[1]
        assert form['is_for_structure'].checked is False

        # on associe ce compte à une structure
        form['is_for_structure'] = True
        form['structure'].select(text="Olieux")
        form.submit(status=302).follow()

        user = StructureUser.objects.get(pk=user.pk)
        assert user.structure == structure

    def test_edit_structure_user_to_user(self):
        structure = StructureFactory(nom="Olieux")
        user = StructureUserFactory(structure=structure)
        url = self.get_edit_url(user.pk)

        form = self.get(url).forms[1]
        assert form['is_for_structure'].checked is True

        # on retire l'association à une structure
        form['is_for_structure'] = False
        form.submit(status=302).follow()

        user = User.objects.get(pk=user.pk)
        assert not StructureUser.objects.filter(pk=user.pk).exists()


class TestUserHomepage(ViewMixin):
    url = WAGTAILHOME_URL

    def test_structure_creation_warning(self, user, structure_user):
        assert "créer votre structure" not in self.get(user=user, status=200)

        html = self.get(user=structure_user, status=200).html
        assert html.find('a', string="créer votre structure")

        structure_user.structure = StructureFactory()
        structure_user.save()

        assert "créer votre structure" not in self.get(user=structure_user)
