import json

from django.contrib.gis.gdal import CoordTransform, SpatialReference
from django.core.serializers.base import SerializerDoesNotExist
from django.core.serializers.json import DjangoJSONEncoder
from django.core.serializers.python import Serializer as PythonSerializer

GEOJSON_SRID = 4326


class Serializer(PythonSerializer):
    """
    Sérialise des objets au format GeoJSON, en suivant la RFC7946.

    Il prend les options supplémentaires suivantes :
    geometry_field -- Le nom du champ géométrique du modèle (défaut : geom)
    as_collection -- Détermine s'il faut retourner un type FeatureCollection
                     même avec un seul objet (défaut : True)
    to_json -- Détermine s'il faut encoder les objets en JSON (défaut : True)

    L’option `fields` peut être utilisée pour limiter les champs qui seront
    présents dans la clé `properties` de chaque objet.

    Voir : https://tools.ietf.org/html/rfc7946
    """

    def start_serialization(self):
        self.geometry_field = self.options.get('geometry_field', 'geom')
        self.as_collection = self.options.get('as_collection', True)
        self.to_json = self.options.get('to_json', True)

        if (
            self.selected_fields is not None
            and self.geometry_field not in self.selected_fields
        ):
            self.selected_fields = [*self.selected_fields, self.geometry_field]

        self._cts = {}
        self._current = None
        self.value = None
        self.features = []

    def end_serialization(self):
        self.value = (
            {'type': 'FeatureCollection', 'features': self.features}
            if self.as_collection or len(self.features) > 1
            else self.features[0]
        )

        if self.to_json:
            json.dump(self.value, self.stream, cls=DjangoJSONEncoder)

    def start_object(self, obj):
        self._current = {}
        self._geometry = None

    def end_object(self, obj):
        self.features.append(self.get_dump_object(obj))
        self._current = None

    def get_dump_object(self, obj):
        data = {
            'type': 'Feature',
            'id': obj._meta.pk.value_to_string(obj),
        }
        if self._current:
            data['properties'] = self._current
        if self._geometry:
            if self._geometry.srid != GEOJSON_SRID:
                if self._geometry.srid not in self._cts:
                    srs = SpatialReference(GEOJSON_SRID)
                    self._cts[self._geometry.srid] = CoordTransform(
                        self._geometry.srs, srs
                    )
                self._geometry.transform(self._cts[self._geometry.srid])
            data['geometry'] = eval(self._geometry.geojson)
        else:
            data['geometry'] = None
        return data

    def handle_field(self, obj, field):
        if field.name == self.geometry_field:
            self._geometry = field.value_from_object(obj)
        else:
            super().handle_field(obj, field)

    def getvalue(self):
        if not self.to_json:
            return self.value
        return super(PythonSerializer, self).getvalue()


class Deserializer:
    def __init__(self, *args, **kwargs):
        raise SerializerDoesNotExist(
            "geojson is a serialization-only serializer"
        )
