from django.urls import reverse
from django.utils.functional import cached_property

from wagtail.contrib.modeladmin.helpers import AdminURLHelper

import pytest
from pytest_django.asserts import assertRedirects

WAGTAILHOME_URL = reverse('wagtailadmin_home')


def assertRedirectsToHome(response, **kwargs):
    assertRedirects(response, WAGTAILHOME_URL, **kwargs)


class ViewMixin:
    """
    Mixin ajoutant des facilitées pour faire des requêtes avec Webtest, à
    appliquer aux classes définissant des tests sur les vues.
    """

    #: L'URL à utiliser par défaut pour les requêtes.
    url = None

    #: L'utilisateur à positionner par défaut dans les requêtes.
    user = None

    #: Détermine s'il faut activer la protection CSRF.
    csrf_checks = True

    @pytest.fixture(autouse=True)
    def setup_django_app(self, django_app_factory):
        self.django_app = django_app_factory(csrf_checks=self.csrf_checks)

    def _query(self, method, url=None, **kwargs):
        """
        Exécute la requête de type `method` vers `url`. Pour les autres
        arguments disponibles, voir `webtest.app.TestApp.get`.
        """
        kwargs.setdefault('user', self.user)
        return getattr(self.django_app, method)(url or self.url, **kwargs)

    def get(self, url=None, **kwargs):
        return self._query('get', url, **kwargs)

    def post(self, url=None, **kwargs):
        return self._query('post', url, **kwargs)


class AdminViewMixin(ViewMixin):
    """
    Étends `ViewMixin` en positionnant un compte administrateur comme
    utilisateur à utiliser dans les requêtes.
    """

    @pytest.fixture(autouse=True)
    def setup_user(self, admin_user):
        self.user = admin_user


class ModelAdminView(AdminViewMixin):
    """
    Définis des utilitaires et des tests communs pour les vues basées sur
    `ModelAdmin` de Wagtail.
    """

    #: Le modèle sur lequel repose la classe.
    model = None

    #: La classe permettant de fabriquer des objets.
    factory = None

    @cached_property
    def url_helper(self):
        return AdminURLHelper(self.model)

    @property
    def index_url(self):
        return self.url_helper.index_url

    @property
    def create_url(self):
        return self.url_helper.create_url

    def get_edit_url(self, pk):
        return self.url_helper.get_action_url('edit', instance_pk=pk)

    def get_delete_url(self, pk):
        return self.url_helper.get_action_url('delete', instance_pk=pk)

    # Tests communs

    def test_index(self):
        self.factory.create_batch(2)

        response = self.get(self.index_url, status=200)
        assert len(response.html.select('[data-object-pk]')) == 2

    def test_delete(self):
        instance = self.factory.create()
        url = self.get_delete_url(instance.pk)

        delete_form = self.get(url, status=200).forms[1]
        assert delete_form.action == url

        delete_form.submit().follow(status=200)
        assert not self.model.objects.filter(pk=instance.pk).exists()


class ViewSetMixin(AdminViewMixin):
    """
    Définit des utilitaires pour les vues basées sur `ViewSet` de Wagtail.
    """

    #: Le modèle sur lequel repose la classe.
    model = None

    #: La classe permettant de fabriquer des objets.
    factory = None

    #: Le namespace utilisé pour ce ViewSet.
    namespace = None

    def get_url_name(self, view_name):
        return '%s:%s' % (self.namespace, view_name)

    @cached_property
    def index_url(self):
        return reverse(self.get_url_name('index'))

    @cached_property
    def create_url(self):
        return reverse(self.get_url_name('add'))

    def get_edit_url(self, pk):
        return reverse(self.get_url_name('edit'), args=(pk,))

    def get_delete_url(self, pk):
        return reverse(self.get_url_name('delete'), args=(pk,))

    # Tests communs

    def test_index(self):
        self.factory.create_batch(2)

        response = self.get(self.index_url, status=200)
        assert len(response.html.select('.listing tbody tr')) == 2

    def test_delete(self):
        instance = self.factory.create()
        url = self.get_delete_url(instance.pk)

        delete_form = self.get(url, status=200).forms[1]
        assert delete_form.action == url

        delete_form.submit().follow(status=200)
        assert not self.model.objects.filter(pk=instance.pk).exists()
