import json

from django.core import serializers
from django.core.serializers.base import SerializerDoesNotExist

import pytest

from danse.core.models import Ville


@pytest.fixture
def ville1():
    return Ville.objects.create(
        nom='Ville 1', code_postal='01234', geom="POINT (0 1)"
    )


@pytest.fixture
def ville2():
    return Ville.objects.create(
        nom='Ville 2', code_postal='01235', geom="POINT (1 2)"
    )


class TestGeoJSONSerializer:
    def serialize(self, queryset=None, **kwargs):
        if queryset is None:
            queryset = Ville.objects.all()
        return serializers.serialize('geojson', queryset, **kwargs)

    def test_basic(self, ville1, ville2):
        SERIALIZED_DATA = {
            'type': 'FeatureCollection',
            'features': [
                {
                    'type': 'Feature',
                    'id': '1',
                    'geometry': {'type': 'Point', 'coordinates': [0.0, 1.0]},
                    'properties': {
                        'nom': 'Ville 1',
                        'code_postal': '01234',
                        'description': '',
                        'visuel': '',
                        'url_wikipedia': '',
                    },
                },
                {
                    'type': 'Feature',
                    'id': '2',
                    'geometry': {'type': 'Point', 'coordinates': [1.0, 2.0]},
                    'properties': {
                        'nom': 'Ville 2',
                        'code_postal': '01235',
                        'description': '',
                        'visuel': '',
                        'url_wikipedia': '',
                    },
                },
            ],
        }

        assert json.loads(self.serialize()) == SERIALIZED_DATA
        assert self.serialize(to_json=False) == SERIALIZED_DATA

    def test_empty_geometry(self):
        Ville.objects.create(nom='City1', code_postal='01234')

        features = self.serialize(to_json=False)['features']
        assert len(features) == 1
        assert features[0]['geometry'] is None

    def test_as_collection(self, ville1, ville2):
        serialized = self.serialize(
            queryset=[ville1, ville2], as_collection=False, to_json=False
        )
        assert serialized['type'] == 'FeatureCollection'
        assert len(serialized['features']) == 2

        serialized = self.serialize(
            queryset=[ville1], as_collection=False, to_json=False
        )
        assert serialized['type'] == 'Feature'

    def test_properties(self, ville1):
        serialized = self.serialize(queryset=[ville1], fields=[], to_json=False)
        assert 'properties' not in serialized['features'][0]

        serialized = self.serialize(
            queryset=[ville1], fields=['nom'], to_json=False
        )
        assert set(serialized['features'][0]['properties'].keys()) == {'nom'}

    def test_transform_srid(self, ville1, ville2):
        ville1.geom.transform(3857)
        assert ville1.geom.wkt == 'POINT (0 111325.1428663851)'
        ville2.geom.transform(3857)
        assert (
            ville2.geom.wkt == 'POINT (111319.49079327357 222684.20850554405)'
        )

        serialized = self.serialize(queryset=[ville1, ville2], to_json=False)
        assert serialized['features'][0]['geometry']['coordinates'] == [
            0.0,
            1.0,
        ]
        assert serialized['features'][1]['geometry']['coordinates'] == [
            1.0,
            2.0,
        ]

    def test_deserialize(self):
        with pytest.raises(SerializerDoesNotExist):
            serializers.deserialize('geojson', '{}')
