from django.contrib.gis import forms
from django.contrib.gis.geos import GEOSGeometry

import pytest
from pytest_django.asserts import assertHTMLEqual

from ..widgets import DurationWidget, GeocodedPointWidget


@pytest.fixture
def geos_point():
    return GEOSGeometry("SRID=4326;POINT(9.052734375 42.451171875)")


class WidgetTest:
    def check_html(self, widget, name, value, html='', attrs=None):
        output = widget.render(name, value, attrs=attrs)
        assertHTMLEqual(output, html)


class TestDurationWidget(WidgetTest):
    widget = DurationWidget()

    def test_media(self):
        assert 'css/duration-input.css' in str(self.widget.media)

    def test_render_empty(self):
        html = (
            '<input type="number" name="duration_0" min="0">'
            '<input type="number" name="duration_1" min="0" max="59" step="5">'
        )
        self.check_html(self.widget, 'duration', '', html=html)
        self.check_html(self.widget, 'duration', None, html=html)

    @pytest.mark.parametrize(
        'value, hours, minutes',
        [
            ('00:01:00', '0', '1'),
            ('01:00:00', '1', '0'),
            ('01:10:00', '1', '10'),
            ('1 01:10:00', '25', '10'),
        ],
    )
    def test_render_with_value(self, value, hours, minutes):
        html = (
            '<input type="number" name="duration_0" value="{hours}" min="0">'
            '<input type="number" name="duration_1" value="{minutes}" min="0" max="59" step="5">'  # noqa: E501
        ).format(hours=hours, minutes=minutes)
        self.check_html(self.widget, 'duration', value, html=html)

    def test_constructor_attrs(self):
        html = (
            '<input type="number" name="duration_0" class="test" min="0">'
            '<input type="number" name="duration_1" class="test" min="0" max="59" step="5">'  # noqa: E501
        )
        widget = DurationWidget(attrs={'class': 'test'})
        self.check_html(widget, 'duration', '', html=html)

    @pytest.mark.parametrize('data', [{}, {'duration_0': '', 'duration_1': ''}])
    def test_value_from_datadict_empty(self, data):
        assert self.widget.value_from_datadict(data, {}, 'duration') is None

    @pytest.mark.parametrize(
        'hours, minutes, value',
        [
            ('0', '0', '00:00:00'),
            ('0', '1', '00:01:00'),
            ('1', '0', '01:00:00'),
            ('1', '10', '01:10:00'),
            ('25', '10', '1 01:10:00'),
        ],
    )
    def test_value_from_datadict(self, hours, minutes, value):
        assert (
            self.widget.value_from_datadict(
                {'duration_0': hours, 'duration_1': minutes}, {}, 'duration'
            )
            == value
        )


class TestGeocodedPointWidget:
    def test_render_no_options(self, geos_point):
        class PointForm(forms.Form):
            p = forms.PointField(widget=GeocodedPointWidget)

        form = PointForm(data={'p': geos_point})
        rendered = form.as_p()

        assert (
            '<script>new GeocodedPoint(geodjango_p, {});</script>'
        ) in rendered
        assert '"javascript:geodjango_p.clearFeatures()"' in rendered

    def test_render_readonly(self):
        class PointForm(forms.Form):
            p = forms.PointField(
                widget=GeocodedPointWidget(options={'readOnly': True})
            )

        form = PointForm()
        rendered = form.as_p()

        assert ('GeocodedPoint(geodjango_p, {"readOnly": true});') in rendered
        assert '"javascript:geodjango_p.clearFeatures()"' not in rendered
