from unittest.mock import patch

import pytest

from ..wikipedia import (
    WIKIPEDIA_PAGE_URL_RE,
    Wikipedia,
    get_page_from_url,
    get_page_image,
    get_page_summary,
)


@pytest.mark.parametrize(
    'url, match',
    [
        ('nope', None),
        ('http://fr.wikipedia.org/wiki/Debian', None),
        ('https://fr.wikipedia.org/w/index.php?title=Debian', None),
        ('https://fr.wikipedia.org/wiki/Debian', ('fr', 'Debian')),
        ('https://fr.wikipedia.org/wiki/Debian_(GNU)', ('fr', 'Debian_(GNU)')),
        # vérifie les mots composés
        (
            "https://fr.wikipedia.org/wiki/Villeneuve-d'Ascq",
            ('fr', "Villeneuve-d'Ascq"),
        ),
        # vérifie les arguments et les identifiants en fin d'URL
        ('https://fr.wikipedia.org/wiki/Debian?pouet', ('fr', 'Debian')),
        ('https://fr.wikipedia.org/wiki/Debian#title', ('fr', 'Debian')),
        # vérifie les accents et leur encodage dans le titre
        ('https://fr.wikipedia.org/wiki/Armentières', ('fr', 'Armentières')),
        (
            'https://fr.wikipedia.org/wiki/Armenti%C3%A8res',
            ('fr', 'Armenti%C3%A8res'),
        ),
    ],
)
def test_wikipedia_page_url_re(url, match):
    result = WIKIPEDIA_PAGE_URL_RE.match(url)
    if match:
        assert result.group('lang', 'title') == match
    else:
        assert result is None


def test_get_page_from_url():
    page = get_page_from_url('https://fr.wikipedia.org/wiki/Debian')
    assert page.title == "Debian"
    assert page.language == 'fr'


def test_get_page_from_url_unquote():
    page = get_page_from_url('https://fr.wikipedia.org/wiki/Armenti%C3%A8res')
    assert page.title == "Armentières"


def test_get_page_from_url_invalid():
    with pytest.raises(ValueError):
        get_page_from_url('nope')


# EXTRACTIONS
# ------------------------------------------------------------------------------

# https://en.wikipedia.org/w/api.php?action=query&prop=pageimages&titles=Debian&piprop=name|original&pilimit=1
PAGE_IMAGE_QUERY_RESPONSE = {
    "batchcomplete": "",
    "query": {
        "pages": {
            "8242": {
                "pageid": 8242,
                "ns": 0,
                "title": "Debian",
                "original": {
                    "source": "https://upload.wikimedia.org/wikipedia/commons/f/fa/Debian10_Gnome.png",  # noqa: E501
                    "width": 1920,
                    "height": 1200,
                },
                "pageimage": "Debian10_Gnome.png",
            }
        }
    },
}

DEBIAN_PAGE_ID = '8242'
DEBIAN_PAGE_IMAGE_QUERY = PAGE_IMAGE_QUERY_RESPONSE['query']['pages'][
    DEBIAN_PAGE_ID
]


@pytest.fixture
def debian_page():
    page = get_page_from_url('https://fr.wikipedia.org/wiki/Debian')
    page.pageid = DEBIAN_PAGE_ID
    return page


def test_get_page_summary(debian_page):
    with patch.object(Wikipedia, 'extracts') as mock_extracts:
        mock_extracts.return_value = (
            "Une introduction de la page accentuée et avec des retours à la "
            "ligne.\nIl seront ensuite doublés.\nPuis formatés en HTML."
        )
        # on récupuère la description formaté en HTML
        summary = get_page_summary(debian_page)
        mock_extracts.assert_called_with(
            debian_page, explaintext=1, exchars=None
        )
        assert summary.count('<p>') == 3

        mock_extracts.reset_mock()
        mock_extracts.return_value = None
        # on limite le nombre de caractères, mais vide
        summary = get_page_summary(debian_page, limit=10)
        mock_extracts.assert_called_with(debian_page, explaintext=1, exchars=10)
        assert summary == ''


def test_get_page_image(debian_page):
    with patch.object(Wikipedia, '_query') as mock_query:
        mock_query.return_value = PAGE_IMAGE_QUERY_RESPONSE
        # on récupère l'image originale et son nom
        assert get_page_image(debian_page) == {
            'name': DEBIAN_PAGE_IMAGE_QUERY['pageimage'],
            'original': DEBIAN_PAGE_IMAGE_QUERY['original'],
        }


def test_get_page_image_error(debian_page):
    with patch.object(Wikipedia, '_query') as mock_query:
        mock_query.return_value = {"query": {"pages": {"-1": {}}}}
        with pytest.raises(ValueError):
            get_page_image(debian_page)

        mock_query.return_value = {"query": {"pages": {DEBIAN_PAGE_ID: {}}}}
        assert get_page_image(debian_page) == {}
