import datetime
import json

from django import forms
from django.contrib.gis.forms import OSMWidget
from django.utils.dateparse import parse_duration
from django.utils.duration import duration_string
from django.utils.safestring import mark_safe

OL_ASSETS_BASE_URL = '//cdn.jsdelivr.net/gh/openlayers/openlayers.github.io@main/dist/en/v6.3.1'  # noqa: E501


class DurationWidget(forms.MultiWidget):
    """
    Sépare une durée en deux composants `NumberInput` : un pour les heures et
    un autre pour les minutes.
    """

    class Media:
        css = {'all': ('css/duration-input.css',)}

    def __init__(self, attrs=None):
        widgets = (
            forms.NumberInput(),
            forms.NumberInput(attrs={'max': 59, 'step': 5}),
        )
        super().__init__(widgets, attrs=attrs)

        # définis des attributs globaux
        self.attrs['min'] = 0

    def value_from_datadict(self, data, files, name):
        hours, minutes = super().value_from_datadict(data, files, name)
        return (
            duration_string(
                datetime.timedelta(
                    hours=int(hours or 0), minutes=int(minutes or 0)
                )
            )
            if hours or minutes
            else None
        )

    def decompress(self, value):
        if value:
            value = parse_duration(value)
            minutes = value.total_seconds() // 60
            hours = int(minutes // 60)
            minutes = int(minutes % 60)
            return [hours, minutes]
        return [None, None]


class GeocodedPointWidget(OSMWidget):
    """
    Affiche un point sur une carte OpenStreetMap, en tentant de
    récupérer ses coordonnées en fonction de la valeur d'autres
    champs.

    Voir `static/gis/js/OLGeocodedPoint.js` pour les options possibles.
    """

    class Media:
        extend = False
        css = {
            'all': (
                '{}/css/ol.css'.format(OL_ASSETS_BASE_URL),
                'gis/css/ol3.css',
            )
        }
        js = (
            '{}/build/ol.js'.format(OL_ASSETS_BASE_URL),
            'gis/js/OLMapWidget.js',
            'gis/js/OLGeocodedPoint.js',
        )

    def __init__(self, options=None, **kwargs):
        super().__init__(**kwargs)
        self.options = options or {}

    def render(self, name, value, attrs=None, renderer=None):
        context = self.get_context(name, value, attrs)

        if self.options.get('readOnly', False):
            context['disabled'] = True

        return mark_safe(
            '{0}<script>new GeocodedPoint({1}, {2});</script>'.format(
                self._render(self.template_name, context, renderer),
                context['module'],
                json.dumps(self.options),
            )
        )
