import re
from urllib.parse import unquote

from django.utils.html import linebreaks

from wikipediaapi import Wikipedia, WikipediaPage

WIKIPEDIA_PAGE_URL_RE = re.compile(
    r'https://(?P<lang>[\w]+)\.wikipedia.org/wiki/(?P<title>[\w\-\'\%\(\)]+)'
)


def get_page_from_url(url: str) -> 'WikipediaPage':
    """Récupère et retourne la page correspondand à l'URL donnée."""
    url_match = WIKIPEDIA_PAGE_URL_RE.match(unquote(url))
    if url_match is None:
        raise ValueError("L'URL doit pointer vers une page Wikipedia.")
    page_title, lang = url_match.group('title', 'lang')
    return Wikipedia(lang).page(page_title)


def get_page_summary(page: 'WikipediaPage', limit: int = 0) -> str:
    """
    Retourne le résumé d'une page optionnellement limité à `limit`
    caractères, avec ses paragraphes formatés en HTML.

    Voir : https://www.mediawiki.org/wiki/Extension:TextExtracts
    """
    if limit <= 0:
        limit = None
    summary = page.wiki.extracts(page, explaintext=1, exchars=limit)
    return linebreaks(summary.replace('\n', '\n\n')) if summary else ''


def get_page_image(page: 'WikipediaPage') -> dict:
    """
    Retourne l'image de la page, calculée par l'extension PageImages et
    visible dans les informations.

    Voir : https://www.mediawiki.org/wiki/Extension:PageImages
    """
    params = {
        'action': 'query',
        'prop': 'pageimages',
        'titles': page.title,
        'piprop': 'name|original',
        'pilimit': 1,
    }
    raw = page.wiki._query(page, params)
    try:
        raw_page = raw['query']['pages'][str(page.pageid)]
    except KeyError:
        raise ValueError("Unable to find pageid in response")
    try:
        return {
            'name': raw_page['pageimage'],
            'original': raw_page['original'],
        }
    except KeyError:
        return {}
