const ERROR_MESSAGES = {
  404: {
    name: 'Élément introuvable',
    message: `L'élément demandé est introuvable.`,
  },
  default: {
    name: 'Erreur serveur',
    message:
      'Une erreur inattendue est survenue en interrogeant le serveur de données.',
  },
};

const NETWORK_ERROR_MESSAGE = `Le serveur de données est injoignable, il est possible que cela soit dû à votre connexion à Internet.`;

/**
 * Formate l'exception correspondant à l'erreur rencontrée lors d'une
 * requête à l'API. Il est possible de surcharger le nom et le message
 * d'erreur à définir en fonction du code de statut de la réponse via
 * `errorMessages`.
 */
export async function formatAPIError(error, errorMessages = {}) {
  let name;
  let message;
  let statusCode;

  const setStatus = (status, responseMessage) => {
    statusCode = status;

    const defaultError = ERROR_MESSAGES[status] || ERROR_MESSAGES.default;
    const defaultMessage = responseMessage || defaultError.message;

    const customMessage = errorMessages[status] || errorMessages.default;

    if (!customMessage) {
      name = defaultError.name;
      message = defaultMessage;
    } else if (typeof customMessage === 'string') {
      name = defaultError.name;
      message = customMessage;
    } else {
      name = customMessage.name || defaultError.name;
      message = customMessage.message || defaultMessage;
    }
  };

  if (error.response) {
    setStatus(error.response.status, (await error.response.json()).detail);
  } else if (error.message && error.message.includes('NetworkError')) {
    setStatus(500, NETWORK_ERROR_MESSAGE);
  } else {
    setStatus(500);
  }

  return {
    ...error,
    name,
    message,
    statusCode,
  };
}

/**
 * Retarde l'exécution de la fonction `func` de `wait` millisecondes,
 * en évitant ses invocations multiples. Seul le dernier appel de la
 * fonction sera exécuté après le délais passé.
 *
 * @link https://github.com/dhershman1/vue-debounce
 */
export function debounce(func, wait) {
  let timeout = null;

  const debounced = function (...args) {
    const later = () => {
      timeout = null;
      func.apply(this, args);
    };

    clearTimeout(timeout);
    timeout = setTimeout(later, wait);

    if (!timeout) {
      later();
    }
  };

  debounced.cancel = () => {
    clearTimeout(timeout);
    timeout = null;
  };

  debounced.pending = () => timeout !== null;

  return debounced;
}

/**
 * Décode les entités HTML d'une chaîne de caractères.
 * @param {String} text
 * @returns {String}
 */
export function decodeHTMLEntities(text) {
  const element = document.createElement('textarea');
  element.innerHTML = text;
  return element.value;
}

/**
 * Retourne un entier choisi aléatoirement entre un minimum (inclus) et un
 * maximum (exclu).
 * @param {Number} min
 * @param {Number} max
 * @returns {Number}
 */
export function getRandomInt(min, max) {
  min = Math.ceil(min);
  max = Math.floor(max);
  return Math.floor(Math.random() * (max - min) + min);
}

/**
 * Crée un object composé des propriétés de `object` pour lesquelles
 * `predicate` passe. Cette fonction est appelée avec la valeur et le nom de
 * la propriété en arguments.
 * @param {Object} object L'objet source.
 * @param {Function} predicate La fonction appelée sur les propriétés.
 * @returns {Object}
 */
export function pickBy(object, predicate) {
  if (object == null) {
    return {};
  }

  return Object.assign(
    {},
    ...Object.keys(object)
      .filter((key) => predicate(object[key], key))
      .map((key) => ({ [key]: object[key] }))
  );
}
