const PRECACHE = 'precache-<%= options.version %>';
const RUNTIME = 'runtime';

const PRECACHE_URLS = [<%= options.offlineAssets.map((i) => `'${i}'`).join(', ') %>];
const ASSETS_URL_PATTERN = new RegExp('<%= options.assetsURLPattern %>');

/**
 * Détermine si le fichier avec l'URL donnée peut être mis en cache, ce
 * qui est le cas si le domaine est le même et que ce n'est pas un appel
 * à l'API.
 * @param {String} url
 * @returns {Boolean}
 */
function canCacheURL(url) {
  if (typeof url === 'string') {
    url = new URL(url);
  }

  return (
    url.origin === self.location.origin &&
    (url.pathname.match(ASSETS_URL_PATTERN) ||
      PRECACHE_URLS.includes(url.pathname))
  );
}

/**
 * Mets en cache quelques fichiers à de l'installation.
 */
self.addEventListener('install', (event) => {
  event.waitUntil(
    caches
      .open(PRECACHE)
      .then((cache) => cache.addAll(PRECACHE_URLS))
  );
});

/**
 * Vide le cache des anciens fichiers à l'activation.
 */
self.addEventListener('activate', (event) => {
  const currentCaches = [PRECACHE, RUNTIME];

  event.waitUntil(
    caches
      .keys()
      .then((cacheNames) =>
        cacheNames.filter((name) => !currentCaches.includes(name))
      )
      .then((cacheNames) => {
        return Promise.all(cacheNames.map((name) => caches.delete(name)));
      })
      .then(() => self.clients.claim())
  );
});

/**
 * Écoute les messages envoyés par l'application, notamment lorsqu'une mise
 * à jour est déclenchée.
 */
self.addEventListener('message', (event) => {
  const replyPort = event.ports[0];

  if (replyPort && event.data && event.data.type === 'skip-waiting') {
    event.waitUntil(
      self.skipWaiting().then(
        () => replyPort.postMessage({ error: null }),
        (error) => replyPort.postMessage({ error }),
      )
    );
  }
});

/**
 * Retourne le contenu depuis le cache s'il existe, sinon depuis le réseau
 * avant de le mettre en cache pour la suite.
 */
self.addEventListener('fetch', (event) => {
  if (!canCacheURL(event.request.url)) {
    return;
  }

  event.respondWith(
    caches.match(event.request).then((cachedResponse) => {
      return (
        cachedResponse ||
        caches.open(RUNTIME).then((cache) => {
          return fetch(event.request).then((response) => {
            return cache
              .put(event.request, response.clone())
              .then(() => response);
          });
        })
      );
    })
  );
});

/**
 * Affiche une notification lors d'un nouvel évènement Push.
 */
self.addEventListener('push', (event) => {
  if (!event.data) {
    return;
  }

  const payload = event.data.json();
  const options = {
    body: payload.body,
    icon: '<%= options.notificationIcon %>',
    badge: '<%= options.notificationBadge %>',
    data: { url: '<%= options.notificationClickURL %>' },
  };

  event.waitUntil(self.registration.showNotification(payload.title, options));
});

/**
 * Ouvre l'application sur la page spécifiée par la notification lors du
 * clique sur celle-ci.
 */
self.addEventListener('notificationclick', (event) => {
  const url = event.notification.data.url;
  event.notification.close();

  event.waitUntil(
    (async () => {
      const clientList = await self.clients.matchAll({
        type: 'window',
        includeUncontrolled: true,
      });
      for (let i = 0; i < clientList.length; i++) {
        const client = clientList[i];
        if (client.url == url && 'focus' in client) {
          return client.focus();
        }
      }
      if (self.clients.openWindow) {
        return self.clients.openWindow(url);
      }
    })()
  );
});
