import api from '~/common/api';
import { getRandomPersonnages } from '~/common/jeu';
import {
  isSubscribedToPush,
  subscribeToPush,
  unsubscribeUserFromPush,
} from '~/common/notifications';

/**
 * Communique avec le Service Worker pour qu'il arrête d'attendre,
 * nécessaire notamment lors de la mise à jour.
 * @param {Worker} worker
 */
function skipWorkerWaiting(worker) {
  if (!worker) return Promise.resolve();
  return new Promise((resolve, reject) => {
    const channel = new MessageChannel();
    channel.port1.onmessage = (event) => {
      if (event.data.error) {
        reject(event.data);
      } else {
        resolve(event.data);
      }
    };
    worker.postMessage({ type: 'skip-waiting' }, [channel.port2]);
  });
}

/**
 * @param {CustomEvent} event
 * @this DansePWAMixin
 */
function onNeedUpdate(event) {
  this.$buefy.snackbar.open({
    message: `Une nouvelle version de l'application est disponible.`,
    type: 'is-primary',
    queue: false,
    indefinite: true,
    cancelText: 'Ignorer',
    actionText: 'Mettre à jour',
    onAction: async () => {
      const registration = event.detail;
      try {
        await skipWorkerWaiting(registration.waiting);
        window.location.reload();
      } catch (e) {
        console.error(e); // eslint-disable-line no-console
      }
    },
  });
}

/**
 * @param {BeforeInstallPromptEvent} event
 * @this DansePWAMixin
 */
function onInstallPrompt(event) {
  event.preventDefault();

  this.$buefy.snackbar
    .open({
      message: `
        <figure class="image is-96x96 is-pulled-left mb-1">
          <img src="${getRandomPersonnages('.small')}" alt="" />
        </figure>
        <div class="title is-5 mb-2 has-text-dark has-text-weight-bold has-text-centered">
          Envie de faire danser votre téléphone&nbsp;?
        </div>
        Installez l'application et retrouvez chaque jour tous les spectacles et ateliers de danse de la région Hauts-de-France. Bienvenue dans la Zone À Danser&nbsp;!
      `,
      type: 'is-primary',
      indefinite: true,
      cancelText: 'Non, merci',
      actionText: 'Installer',
      onAction: () => {
        event.prompt();
        event.userChoice.then((choiceResult) => {
          if (choiceResult.outcome === 'accepted') {
            this.subscribeToPush();

            if (this.$matomo) {
              this.$matomo.trackEvent('Application', 'Installation');
            }
          }
        });
      },
    })
    .$on('close', () => {
      this.$store.dispatch('user/update', { showInstallPrompt: false });
    });
}

// @vue/component
const DansePWAMixin = {
  data() {
    return {
      isSubscribedToPush: null,
      isStandalone: null,
    };
  },
  async created() {
    this.isSubscribedToPush = await isSubscribedToPush();
  },
  mounted() {
    const standaloneMediaQuery = window.matchMedia(
      '(display-mode: standalone)'
    );
    standaloneMediaQuery.addEventListener('change', (event) => {
      this.isStandalone = event.matches;
    });

    // Définis l'état initial depuis `navigator` également pour supporter
    // les navigateurs qui ne définissent pas `display-mode: standalone`
    this.isStandalone = navigator.standalone || standaloneMediaQuery.matches;

    // Écoute les mises à jour disponibles, émis par le Service Worker
    document.addEventListener('needupdate', onNeedUpdate.bind(this));

    // Affiche un bandeau pour installer l'application quand le navigateur le
    // supporte (ce qui n'est pas le cas de Firefox)
    if (this.$store.state.user.showInstallPrompt) {
      window.addEventListener(
        'beforeinstallprompt',
        onInstallPrompt.bind(this)
      );
    }
  },
  methods: {
    /**
     * Abonne le client au service Push de l'application.
     */
    async subscribeToPush() {
      if (this.isSubscribedToPush) {
        return true;
      }

      const serverKey = await this.$http
        .$get(api.Notifications.INFO)
        .then((data) => data.server_key);
      const subscription = await subscribeToPush(serverKey);

      await this.$http.$post(api.Notifications.SUBSCRIBE, {
        subscription: subscription.toJSON(),
      });

      this.isSubscribedToPush = true;
    },
    /**
     * Désabonne le client du service Push de l'application.
     */
    async unsubscribeFromPush() {
      const endpoint = await unsubscribeUserFromPush();

      this.isSubscribedToPush = false;

      if (endpoint) {
        this.$http.$post(api.Notifications.UNSUBSCRIBE, { endpoint });
      }
    },
  },
};

export default ({ app }) => {
  if (!app.mixins) app.mixins = [];
  app.mixins.push(DansePWAMixin);
};
