import Vue from 'vue';

/**
 * Met en majuscule la première lettre d'une chaîne de caractères.
 * @param {string} value
 * @returns {string}
 */
export function capfirst(value) {
  return value.toString().charAt(0).toUpperCase() + value.toString().slice(1);
}

/**
 * Formate un montant pour la devise française.
 * @param {(string|number)} value
 * @param {string} [empty=''] - La valeur retournée si le montant est nul.
 * @returns {string}
 */
export function currency(value, empty = '') {
  if (typeof value === 'string') {
    value = Number(value);
  }

  return !isNaN(value) && value
    ? new Intl.NumberFormat('fr-FR', {
        style: 'currency',
        currency: 'EUR',
        minimumFractionDigits: 0,
      }).format(value)
    : empty;
}

function _toString(value, pad = false) {
  return pad ? value.toString().padStart(2, '0') : value.toString();
}

/**
 * Formate une durée exprimée en secondes.
 * @param {(string|number)} value
 * @param {string} [empty=''] - La valeur retournée si la durée est nulle.
 * @returns {string}
 */
export function duration(value, empty = '') {
  if (typeof value !== 'number') {
    value = Number(value);
  }

  if (isNaN(value) || !value) {
    return empty;
  }

  value = Math.abs(value);

  const hours = Math.floor(value / 3600);
  const minutes = Math.floor((value % 3600) / 60);
  const seconds = Math.floor(value % 60);

  const parts = [];

  if (seconds) {
    if (hours) {
      parts.push(`${_toString(hours)}h`);

      if (minutes) {
        parts.push(`${_toString(minutes, true)}min`);
      }
    } else if (minutes) {
      parts.push(`${_toString(minutes)}min`);
    }

    parts.push(`${_toString(seconds, !!parts.length)}s`);
  } else if (hours) {
    parts.push(`${_toString(hours)}h`);

    if (minutes) {
      parts.push(`${_toString(minutes, true)}`);
    }
  } else if (minutes) {
    parts.push(`${_toString(minutes)}min`);
  }

  return parts.length ? parts.join('') : empty;
}

/**
 * Formate des coordonnées en une URI geo suivant la RFC5870.
 * @param {string} value
 * @param {string} [empty=''] - La valeur retournée si la position est nulle.
 * @link https://tools.ietf.org/html/rfc5870
 */
export function geocoords(value, empty = '') {
  if (!value || !value.lat || !value.lon) {
    return empty;
  }

  return `geo:${value.lat},${value.lon}`;
}

/**
 * Extrait le nom d'hôte d'une URL.
 * @param {string} value
 * @param {boolean} [strip=true] - S'il faut retirer le préfixe "www.".
 * @returns {string}
 */
export function hostname(value, strip = true) {
  let hostname;

  try {
    hostname = new URL(value).hostname;
  } catch (e) {
    hostname = value.split('/', 1)[0];
  }

  return strip && hostname.startsWith('www.')
    ? hostname.substring(4)
    : hostname;
}

export function pluralize(word, amount, plural = null) {
  return amount > 1 || amount === 0 ? plural || `${word}s` : word;
}

/**
 * Formate un code postal.
 * @param {string} value
 * @param {boolean} [departmentOnly=true] - S'il faut retourner que le département.
 * @returns {string}
 */
export function postcode(value, departmentOnly = false) {
  const department = value.slice(0, 2);
  return departmentOnly ? department : `${department} ${value.slice(2)}`;
}

Vue.filter('capfirst', capfirst);
Vue.filter('currency', currency);
Vue.filter('duration', duration);
Vue.filter('geocoords', geocoords);
Vue.filter('hostname', hostname);
Vue.filter('pluralize', pluralize);
Vue.filter('postcode', postcode);
